@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "Chan.h"
 #include "znc.h"
@@ -21,8 +27,10 @@ CChan::CChan(const CString& sName, CUser* pUser, bool bInConfig) {
 	m_bDetached = false;
 	m_uBufferCount = m_pUser->GetBufferCount();
 	m_bKeepBuffer = m_pUser->KeepBuffer();
+	m_bDisabled = false;
 	Reset();
 }
+
 CChan::~CChan() {
 	ClearNicks();
 }
@@ -40,6 +48,7 @@ void CChan::Reset() {
 	m_ulTopicDate = 0;
 	m_ulCreationDate = 0;
 	ClearNicks();
+	ResetJoinTries();
 }
 
 bool CChan::WriteConfig(CFile& File) {
@@ -104,7 +113,7 @@ void CChan::JoinUser(bool bForce, const CString& sKey, CClient* pClient) {
 					sPerm += c;
 				}
 			}
-			if(pThisClient->HasUHNames() && !a->second->GetIdent().empty() && a->second->GetHost().empty()) {
+			if(pThisClient->HasUHNames() && !a->second->GetIdent().empty() && !a->second->GetHost().empty()) {
 				sNick = a->first + "!" + a->second->GetIdent() + "@" + a->second->GetHost();
 			} else {
 				sNick = a->first;
@@ -155,7 +164,7 @@ CString CChan::GetModeString() const {
 		}
 	}
 
-	return (sModes.empty()) ? sModes : ("+" + sModes + sArgs);
+	return sModes.empty() ? sModes : CString("+" + sModes + sArgs);
 }
 
 CString CChan::GetModeForNames() const {
@@ -422,7 +431,7 @@ bool CChan::AddNick(const CString& sNick) {
 	}
 
 	sTmp = p;
-	sIdent = sTmp.Token(1, true, "!").Token(0, true, "@");
+	sIdent = sTmp.Token(1, true, "!").Token(0, false, "@");
 	sHost = sTmp.Token(1, true, "@");
 	sTmp = sTmp.Token(0, false, "!");
 
