@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _CHAN_H
 #define _CHAN_H
@@ -111,6 +117,10 @@ public:
 	void SetDetached(bool b = true) { m_bDetached = b; }
 	void SetInConfig(bool b) { m_bInConfig = b; }
 	void SetCreationDate(unsigned long u) { m_ulCreationDate = u; }
+	void Disable() { m_bDisabled = true; }
+	void Enable() { m_bDisabled = false; }
+	void IncJoinTries() { m_uJoinTries++; }
+	void ResetJoinTries() { m_uJoinTries = 0; }
 	// !Setters
 
 	// Getters
@@ -118,7 +128,7 @@ public:
 	CString GetOptions() const;
 	CString GetModeArg(unsigned char uMode) const;
 	unsigned int GetPermCount(unsigned char uPerm);
-	const bool IsOn() const { return m_bIsOn; }
+	bool IsOn() const { return m_bIsOn; }
 	const CString& GetName() const { return m_sName; }
 	const map<unsigned char, CString>& GetModes() const { return m_musModes; }
 	const CString& GetKey() const { return (!m_sCurKey.empty()) ? m_sCurKey : m_sKey; }
@@ -137,6 +147,8 @@ public:
 	bool IsDetached() const { return m_bDetached; }
 	bool InConfig() const { return m_bInConfig; }
 	unsigned long GetCreationDate() const { return m_ulCreationDate; }
+	bool IsDisabled() const { return m_bDisabled; }
+	unsigned int GetJoinTries() { return m_uJoinTries; }
 	// !Getters
 private:
 protected:
@@ -146,6 +158,7 @@ protected:
 	bool				m_bKeepBuffer;
 	bool				m_bAutoCycle;
 	bool				m_bInConfig;
+	bool				m_bDisabled;
 	CString				m_sName;
 	CString				m_sKey;
 	CString				m_sCurKey;
@@ -156,6 +169,7 @@ protected:
 	CUser*				m_pUser;
 	CNick				m_Nick;
 	unsigned int		m_uLimit;
+	unsigned int		m_uJoinTries;
 	CString				m_sDefaultModes;
 	vector<CString>		m_vsBans;
 	map<CString,CNick*>	m_msNicks;	// Todo: make this caseless (irc style)
