@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "Client.h"
@@ -9,6 +15,17 @@
 #include "DCCSock.h"
 #include "Server.h"
 
+CClient::~CClient() {
+	if (!m_spAuth.IsNull()) {
+		CClientAuth* pAuth = (CClientAuth*) &(*m_spAuth);
+		pAuth->SetClient(NULL);
+	}
+	if (m_pUser != NULL) {
+		m_pUser->AddBytesRead(GetBytesRead());
+		m_pUser->AddBytesWritten(GetBytesWritten());
+	}
+}
+
 void CClient::ReadLine(const CString& sData) {
 	CString sLine = sData;
 
@@ -39,9 +56,9 @@ void CClient::ReadLine(const CString& sData) {
 			if ((m_bGotNick) && (m_bGotUser)) {
 				AuthUser();
 			}
-		}
 
-		return;		// Don't forward this msg.  ZNC has already registered us.
+			return;		// Don't forward this msg.  ZNC has already registered us.
+		}
 	} else if (sCommand.CaseCmp("NICK") == 0) {
 		CString sNick = sLine.Token(1);
 		if (sNick.Left(1) == ":") {
@@ -58,6 +75,11 @@ void CClient::ReadLine(const CString& sData) {
 			return;		// Don't forward this msg.  ZNC will handle nick changes until auth is complete
 		}
 
+		if (!m_pIRCSock) {
+			// No need to check against IRC nick or to forward it
+			return;
+		}
+
 		if ((m_pUser) && (sNick.CaseCmp(m_pUser->GetNick()) == 0)) {
 			m_uKeepNickCounter++;
 			// If the user is changing his nick to the conifg nick, set keepnick to the config value
@@ -75,20 +97,27 @@ void CClient::ReadLine(const CString& sData) {
 			}
 		}
 	} else if (sCommand.CaseCmp("USER") == 0) {
-		if ((!m_bAuthed) && (m_sUser.empty())) {
-			m_sUser = sLine.Token(1);
+		if (!m_bAuthed) {
+			if (m_sUser.empty()) {
+				m_sUser = sLine.Token(1);
+			}
+
+			m_bGotUser = true;
+
+			if ((m_bGotPass) && (m_bGotNick)) {
+				AuthUser();
+			} else if (!m_bGotPass) {
+				PutClient(":irc.znc.com NOTICE AUTH :*** "
+					"You need to send your password. "
+					"Try /quote PASS <username>:<password>");
+			}
+
+			return;		// Don't forward this msg.  ZNC has already registered us.
 		}
-
-		m_bGotUser = true;
-
-		if ((m_bGotPass) && (m_bGotNick)) {
-			AuthUser();
-		}
-
-		return;		// Don't forward this msg.  ZNC has already registered us.
 	}
 
 	if (!m_pUser) {
+		PutClient("ERROR :You need to send your password first!");
 		Close();
 		return;
 	}
@@ -282,7 +311,7 @@ void CClient::ReadLine(const CString& sData) {
 		CChan* pChan = m_pUser->FindChan(sTarget);
 
 		if ((pChan) && (pChan->KeepBuffer())) {
-			pChan->AddBuffer(":" + GetNickMask() + " NOTICE " + sTarget + " :" + sMsg);
+			pChan->AddBuffer(":" + GetNickMask() + " NOTICE " + sTarget + " :" + m_pUser->AddTimestamp(sMsg));
 		}
 
 		// Relay to the rest of the clients that may be connected to this user
@@ -426,8 +455,12 @@ void CClient::ReadLine(const CString& sData) {
 			CChan* pChan = m_pUser->FindChan(sTarget);
 
 			if (sCTCP.Token(0).CaseCmp("ACTION") == 0) {
+				CString sMessage = sCTCP.Token(1, true);
+				MODULECALL(OnUserAction(sTarget, sMessage), m_pUser, this, return);
+				sCTCP = "ACTION " + sMessage;
+
 				if (pChan && pChan->KeepBuffer()) {
-					pChan->AddBuffer(":" + GetNickMask() + " PRIVMSG " + sTarget + " :\001" + sCTCP + "\001");
+					pChan->AddBuffer(":" + GetNickMask() + " PRIVMSG " + sTarget + " :\001ACTION " + m_pUser->AddTimestamp(sMessage) + "\001");
 				}
 
 				// Relay to the rest of the clients that may be connected to this user
@@ -442,10 +475,8 @@ void CClient::ReadLine(const CString& sData) {
 						}
 					}
 				}
-#ifdef _MODULES
 			} else {
 				MODULECALL(OnUserCTCP(sTarget, sCTCP), m_pUser, this, return);
-#endif
 			}
 
 			PutIRC("PRIVMSG " + sTarget + " :\001" + sCTCP + "\001");
@@ -496,7 +527,7 @@ void CClient::ReadLine(const CString& sData) {
 		CChan* pChan = m_pUser->FindChan(sTarget);
 
 		if ((pChan) && (pChan->KeepBuffer())) {
-			pChan->AddBuffer(":" + GetNickMask() + " PRIVMSG " + sTarget + " :" + sMsg);
+			pChan->AddBuffer(":" + GetNickMask() + " PRIVMSG " + sTarget + " :" + m_pUser->AddTimestamp(sMsg));
 		}
 
 		PutIRC("PRIVMSG " + sTarget + " :" + sMsg);
@@ -753,13 +784,42 @@ void CClient::UserCommand(const CString& sLine) {
 		usleep(100000);	// Sleep for 10ms to attempt to allow the previous Broadcast() to go through to all users
 
 		throw CException(CException::EX_Shutdown);
-	} else if (sCommand.CaseCmp("JUMP") == 0) {
+	} else if (sCommand.CaseCmp("JUMP") == 0 ||
+			sCommand.CaseCmp("CONNECT") == 0) {
 		if (m_pUser) {
 			if (m_pIRCSock) {
-				m_pIRCSock->Close();
+				m_pIRCSock->Quit();
 				PutStatus("Jumping to the next server in the list...");
+			} else {
+				PutStatus("Connecting...");
+			}
+
+			m_pUser->SetIRCConnectEnabled(true);
+			m_pUser->CheckIRCConnect();
+			return;
+		}
+	} else if (sCommand.CaseCmp("DISCONNECT") == 0) {
+		if (m_pUser) {
+			if (m_pIRCSock)
+				m_pIRCSock->Quit();
+			m_pUser->SetIRCConnectEnabled(false);
+			PutStatus("Disconnected from IRC. Use 'connect' to reconnect.");
+			return;
+		}
+	} else if (sCommand.CaseCmp("ENABLECHAN") == 0) {
+		CString sChan = sLine.Token(1, true);
+
+		if (sChan.empty()) {
+			PutStatus("Usage: EnableChan <channel>");
+		} else {
+			CChan* pChan = m_pUser->FindChan(sChan);
+			if (!pChan) {
+				PutStatus("Channel [" + sChan + "] not found.");
 				return;
 			}
+			
+			pChan->Enable();
+			PutStatus("Channel [" + sChan + "] enabled.");
 		}
 	} else if (sCommand.CaseCmp("LISTCHANS") == 0) {
 		if (m_pUser) {
@@ -790,7 +850,7 @@ void CClient::UserCommand(const CString& sLine) {
 				CChan* pChan = vChans[a];
 				Table.AddRow();
 				Table.SetCell("Name", pChan->GetPermStr() + pChan->GetName());
-				Table.SetCell("Status", ((vChans[a]->IsOn()) ? ((vChans[a]->IsDetached()) ? "Detached" : "Joined") : "Trying"));
+				Table.SetCell("Status", ((vChans[a]->IsOn()) ? ((vChans[a]->IsDetached()) ? "Detached" : "Joined") : ((vChans[a]->IsDisabled()) ? "Disabled" : "Trying")));
 				Table.SetCell("Conf", CString((pChan->InConfig()) ? "yes" : ""));
 				Table.SetCell("Buf", CString((pChan->KeepBuffer()) ? "*" : "") + CString(pChan->GetBufferCount()));
 				Table.SetCell("Modes", pChan->GetModeString());
@@ -1316,6 +1376,50 @@ void CClient::UserCommand(const CString& sLine) {
 		pChan->SetBufferCount(uLineCount);
 
 		PutStatus("BufferCount for [" + sChan + "] set to [" + CString(pChan->GetBufferCount()) + "]");
+        } else if (m_pUser->IsAdmin() && sCommand.CaseCmp("TRAFFIC") == 0) {
+                CZNC::Get().UpdateTrafficStats();
+                const map<CString, CUser*>& msUsers = CZNC::Get().GetUserMap();
+                CTable Table;
+                Table.AddColumn("Username");
+                Table.AddColumn("In");
+                Table.AddColumn("Out");
+                Table.AddColumn("Total");
+                unsigned long long users_total_in = 0;
+                unsigned long long users_total_out = 0;
+                for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+                    Table.AddRow();
+                    Table.SetCell("Username", it->first);
+                    Table.SetCell("In", CString(it->second->BytesRead()));
+                    Table.SetCell("Out", CString(it->second->BytesWritten()));
+                    Table.SetCell("Total", CString(it->second->BytesRead() + it->second->BytesWritten()));
+                    users_total_in += it->second->BytesRead();
+                    users_total_out += it->second->BytesWritten();
+                }
+                Table.AddRow();
+                Table.SetCell("Username", "<Users>");
+                Table.SetCell("In", CString(users_total_in));
+                Table.SetCell("Out", CString(users_total_out));
+                Table.SetCell("Total", CString(users_total_in + users_total_out));
+
+                Table.AddRow();
+                Table.SetCell("Username", "<ZNC>");
+                Table.SetCell("In", CString(CZNC::Get().BytesRead()));
+                Table.SetCell("Out", CString(CZNC::Get().BytesWritten()));
+                Table.SetCell("Total", CString(CZNC::Get().BytesRead() + CZNC::Get().BytesWritten()));
+
+                Table.AddRow();
+                Table.SetCell("Username", "<Total>");
+                Table.SetCell("In", CString(users_total_in + CZNC::Get().BytesRead()));
+                Table.SetCell("Out", CString(users_total_out + CZNC::Get().BytesWritten()));
+                Table.SetCell("Total", CString(users_total_in + CZNC::Get().BytesRead() + users_total_out + CZNC::Get().BytesWritten()));
+
+                if (Table.size()) {
+                    unsigned int uTableIdx = 0;
+                    CString sLine;
+                    while (Table.GetLine(uTableIdx++, sLine)) {
+                        PutStatus(sLine);
+                    }
+                }
 	} else {
 		PutStatus("Unknown command [" + sCommand + "] try 'Help'");
 	}
@@ -1360,6 +1464,8 @@ void CClient::HelpUser() {
 	Table.AddRow(); Table.SetCell("Command", "SetBuffer");	Table.SetCell("Arguments", "<#chan> [linecount]");	Table.SetCell("Description", "Set the buffer count for a channel");
 	Table.AddRow(); Table.SetCell("Command", "SetVHost");	Table.SetCell("Arguments", "<vhost (ip preferred)>");Table.SetCell("Description", "Set the VHost for this connection");
 	Table.AddRow(); Table.SetCell("Command", "Jump");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Jump to the next server in the list");
+	Table.AddRow(); Table.SetCell("Command", "Disconnect");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Disconnect from IRC");
+	Table.AddRow(); Table.SetCell("Command", "Connect");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Reconnect to IRC");
 	Table.AddRow(); Table.SetCell("Command", "Send");		Table.SetCell("Arguments", "<nick> <file>");		Table.SetCell("Description", "Send a shell file to a nick on IRC");
 	Table.AddRow(); Table.SetCell("Command", "Get");		Table.SetCell("Arguments", "<file>");				Table.SetCell("Description", "Send a shell file to yourself");
 
@@ -1469,6 +1575,8 @@ void CClient::AcceptLogin(CUser& User) {
 	m_pUser = &User;
 
 	if (!m_pUser->IsHostAllowed(GetRemoteIP())) {
+		PutClient(":irc.znc.com 463 " + GetNick() + " :Your host ("
+				+ GetRemoteIP() + ") is not allowed");
 		Close();
 		return;
 	}
@@ -1563,10 +1671,10 @@ void CClient::PutModule(const CString& sModule, const CString& sLine) {
 	Write(":" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.com PRIVMSG " + GetNick() + " :" + sLine + "\r\n");
 }
 
-CString CClient::GetNick() const {
+CString CClient::GetNick(bool bAllowIRCNick) const {
 	CString sRet;
 
-	if ((m_bAuthed) && (m_pIRCSock)) {
+	if ((bAllowIRCNick) && (m_bAuthed) && (m_pIRCSock)) {
 		sRet = m_pIRCSock->GetNick();
 	}
 
