@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _CLIENT_H
 #define _CLIENT_H
@@ -60,21 +66,16 @@ protected:
 class CClient : public Csock {
 public:
 	CClient() : Csock() {
-		Init();
+		InitClient();
 	}
 
 	CClient(const CString& sHostname, unsigned short uPort, int iTimeout = 60) : Csock(sHostname, uPort, iTimeout) {
-		Init();
+		InitClient();
 	}
 
-	virtual ~CClient() {
-		if (!m_spAuth.IsNull()) {
-			CClientAuth* pAuth = (CClientAuth*) &(*m_spAuth);
-			pAuth->SetClient(NULL);
-		}
-	}
+	virtual ~CClient();
 
-	void Init() {
+	void InitClient() {
 		m_pUser = NULL;
 		m_pIRCSock = NULL;
 		m_bAuthed = false;
@@ -90,7 +91,7 @@ public:
 	void AcceptLogin(CUser& User);
 	void RefuseLogin(const CString& sReason);
 
-	CString GetNick() const;
+	CString GetNick(bool bAllowIRCNick = true) const;
 	CString GetNickMask() const;
 	bool HasNamesx() const { return m_bNamesx; }
 	bool HasUHNames() const { return m_bUHNames; }
@@ -120,6 +121,7 @@ public:
 	virtual Csock* GetSockObj(const CString& sHost, unsigned short uPort);
 
 	void SetNick(const CString& s);
+	CUser* GetUser() const { return m_pUser; }
 private:
 protected:
 	bool		m_bAuthed;
