@@ -28,10 +28,15 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.21 $
+* $Revision: 1.61 $
 */
 
 #include "Csocket.h"
+#ifdef __NetBSD__
+#include <sys/param.h>
+#endif /* __NetBSD__ */
+
+#define CS_SRANDBUFFER 128
 
 using namespace std;
 
@@ -40,8 +45,94 @@ namespace Csocket
 {
 #endif /* _NO_CSOCKET_NS */
 
+static int g_iCsockSSLIdx = 0; //!< this get setup once in InitSSL
+int GetCsockClassIdx()
+{
+	return( g_iCsockSSLIdx );
+}
+
+#ifdef HAVE_LIBSSL
+Csock *GetCsockFromCTX( X509_STORE_CTX *pCTX )
+{
+	Csock *pSock = NULL;
+	SSL *pSSL = (SSL *)X509_STORE_CTX_get_ex_data( pCTX, SSL_get_ex_data_X509_STORE_CTX_idx() );
+	if( pSSL )
+		pSock = (Csock *)SSL_get_ex_data( pSSL, GetCsockClassIdx() );
+	return( pSock );
+}
+#endif /* HAVE_LIBSSL */
+
+
+#ifndef HAVE_IPV6
+
+// this issue here is getaddrinfo has a significant behavior difference when dealing with round robin dns on an
+// ipv4 network. This is not desirable IMHO. so when this is compiled without ipv6 support backwards compatibility
+// is maintained.
+
+static int __GetHostByName( const CS_STRING & sHostName, struct in_addr *paddr, u_int iNumRetries )
+{
+	int iReturn = HOST_NOT_FOUND;
+	struct hostent *hent = NULL;
+#ifdef __linux__
+	char hbuff[2048];
+	struct hostent hentbuff;
+
+	int err;
+	for( u_int a = 0; a < iNumRetries; a++ )
+	{
+		memset( (char *)hbuff, '\0', 2048 );
+		iReturn = gethostbyname_r( sHostName.c_str(), &hentbuff, hbuff, 2048, &hent, &err );
+
+		if ( iReturn == 0 )
+			break;
+
+		if ( iReturn != TRY_AGAIN )
+		{
+			CS_DEBUG( "gethostyname_r: " << hstrerror( h_errno ) );
+			break;
+		}
+
+	}
+	if ( ( !hent ) && ( iReturn == 0 ) )
+		iReturn = HOST_NOT_FOUND;
+#else
+	for( u_int a = 0; a < iNumRetries; a++ )
+	{
+		iReturn = HOST_NOT_FOUND;
+		hent = gethostbyname( sHostName.c_str() );
+
+		if ( hent )
+		{
+			iReturn = 0;
+			break;
+		}
+
+		if( h_errno != TRY_AGAIN )
+		{
+			CS_DEBUG( "gethostyname: " << hstrerror( h_errno ) );
+			break;
+		}
+	}
+
+#endif /* __linux__ */
+
+	if ( iReturn == 0 )
+		memcpy( &paddr->s_addr, hent->h_addr_list[0], sizeof( paddr->s_addr ) );
+
+	return( iReturn );
+}
+#endif /* !HAVE_IPV6 */
+
 int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockAddr )
 {
+#ifndef HAVE_IPV6
+	if( pSock )
+		pSock->SetIPv6( false );
+	csSockAddr.SetIPv6( false );
+	if( __GetHostByName( sHostname, csSockAddr.GetAddr(), 3 ) == 0 )
+		return( 0 );
+	
+#else /* HAVE_IPV6 */
 	struct addrinfo *res = NULL;
 	struct addrinfo hints;
 	memset( (struct addrinfo *)&hints, '\0', sizeof( hints ) );
@@ -54,7 +145,10 @@ int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockA
 		return( EAGAIN );
 	else if( ( iRet == 0 ) && ( res ) )
 	{
-		bool bFoundEntry = false;
+		struct addrinfo *pUseAddr = NULL;
+#ifdef __RANDOMIZE_SOURCE_ADDRESSES
+		std::vector< struct addrinfo *> vHostCanidates;
+#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
 		for( struct addrinfo *pRes = res; pRes; pRes = pRes->ai_next )
 		{
 #ifdef __sun
@@ -69,31 +163,67 @@ int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockA
 
 			if( pRes->ai_family == AF_INET )
 			{
-				if( pSock )
-					pSock->SetIPv6( false );
-				csSockAddr.SetIPv6( false );
-				struct sockaddr_in *pTmp = (struct sockaddr_in *)pRes->ai_addr;
-				memcpy( csSockAddr.GetAddr(), &(pTmp->sin_addr), sizeof( *(csSockAddr.GetAddr()) ) );
+#ifdef __RANDOMIZE_SOURCE_ADDRESSES
+				vHostCanidates.push_back( pRes );
+#else
+				pUseAddr = pRes;
+#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
 			}
-#ifdef HAVE_IPV6
 			else if( pRes->ai_family == AF_INET6 )
 			{
-				if( pSock )
-					pSock->SetIPv6( true );
-				csSockAddr.SetIPv6( true );
-				struct sockaddr_in6 *pTmp = (struct sockaddr_in6 *)pRes->ai_addr;
-				memcpy( csSockAddr.GetAddr6(), &(pTmp->sin6_addr), sizeof( *(csSockAddr.GetAddr6()) ) );
+#ifdef __RANDOMIZE_SOURCE_ADDRESSES
+				vHostCanidates.push_back( pRes );
+#else
+				pUseAddr = pRes;
+#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
+			}
+		}
+#ifdef __RANDOMIZE_SOURCE_ADDRESSES
+		if( vHostCanidates.size() > 1 )
+		{
+			// this is basically where getaddrinfo() sorts the list of results. basically to help out what round robin dns does,
+			// pick a random canidate to make this work
+			struct random_data cRandData;
+			char chState[CS_SRANDBUFFER];
+			int32_t iNumber = 0; 
+			if( initstate_r( (u_int)millitime(), chState, CS_SRANDBUFFER, &cRandData ) == 0 && random_r( &cRandData, &iNumber ) == 0 )
+			{
+		   		iNumber %= (int)vHostCanidates.size();
+		   		pUseAddr = vHostCanidates[iNumber];
 			}
-#endif /* HAVE_IPV6 */
 			else
-				continue;
-			bFoundEntry = true;
-			break;
+			{
+				CS_DEBUG( "initstate_r/random_r failed" );
+				pUseAddr = vHostCanidates[0];
+			}
+		}
+		else if( vHostCanidates.size() )
+		{
+			pUseAddr = vHostCanidates[0];
+		}
+#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
+
+		if( pUseAddr && pUseAddr->ai_family == AF_INET )
+		{
+			if( pSock )
+				pSock->SetIPv6( false );
+			csSockAddr.SetIPv6( false );
+			struct sockaddr_in *pTmp = (struct sockaddr_in *)pUseAddr->ai_addr;
+			memcpy( csSockAddr.GetAddr(), &(pTmp->sin_addr), sizeof( *(csSockAddr.GetAddr()) ) );
+		}
+		else if( pUseAddr )
+		{
+			if( pSock )
+				pSock->SetIPv6( true );
+			csSockAddr.SetIPv6( true );
+			struct sockaddr_in6 *pTmp = (struct sockaddr_in6 *)pUseAddr->ai_addr;
+			memcpy( csSockAddr.GetAddr6(), &(pTmp->sin6_addr), sizeof( *(csSockAddr.GetAddr6()) ) );
 		}
 		freeaddrinfo( res );
-		if( bFoundEntry )
+		if( pUseAddr ) // the data pointed to here is invalid now, but the pointer itself is a good test
 			return( 0 );
 	}
+#endif /* ! HAVE_IPV6 */
 	return( ETIMEDOUT );
 }
 
@@ -239,6 +369,9 @@ bool InitSSL( ECompType eCompressionType )
 			SSL_COMP_add_compression_method( CT_RLE, cm );
 	}
 
+	// setting this up once in the begining
+	g_iCsockSSLIdx = SSL_get_ex_new_index( 0, (void *)"CsockGlobalIndex", NULL, NULL, NULL);
+
 	return( true );
 }
 
@@ -257,17 +390,17 @@ void SSLErrors( const char *filename, u_int iLineNum )
 }
 #endif /* HAVE_LIBSSL */
 
-void __Perror( const CS_STRING & s )
+void __Perror( const CS_STRING & s, const char *pszFile, unsigned int iLineNo )
 {
-#if defined(__sun) || defined(_WIN32)
-	CS_DEBUG( s << ": " << strerror( GetSockError() ) );
+#if defined(__sun) || defined(_WIN32) || __NetBSD_Version__ < 4000000000
+	std::cerr << s << "(" << pszFile << ":" << iLineNo << "): " << strerror( GetSockError() ) << endl;
 #else
 	char buff[512];
 	memset( (char *)buff, '\0', 512 );
 	if ( strerror_r( GetSockError(), buff, 511 ) == 0 )
-		CS_DEBUG( s << ": " << buff );
+		std::cerr << s << "(" << pszFile << ":" << iLineNo << "): " << buff << endl;
 	else
-		CS_DEBUG( s << ": Unknown Error Occured" );
+		std::cerr << s << "(" << pszFile << ":" << iLineNo << "): Unknown Error Occured " << endl;
 #endif /* __sun */
 }
 
@@ -365,6 +498,9 @@ Csock::Csock( int itimeout )
 #ifdef HAVE_LIBSSL
 	m_pCerVerifyCB = NULL;
 #endif /* HAVE_LIBSSL */
+#ifdef ___DO_THREADS
+	m_pResolver = NULL;
+#endif /* ___DO_THREADS */
 	Init( "", 0, itimeout );
 }
 
@@ -373,6 +509,9 @@ Csock::Csock( const CS_STRING & sHostname, u_short iport, int itimeout )
 #ifdef HAVE_LIBSSL
 	m_pCerVerifyCB = NULL;
 #endif /* HAVE_LIBSSL */
+#ifdef ___DO_THREADS
+	m_pResolver = NULL;
+#endif /* ___DO_THREADS */
 	Init( sHostname, iport, itimeout );
 }
 
@@ -391,18 +530,24 @@ Csock *Csock::GetSockObj( const CS_STRING & sHostname, u_short iPort )
 Csock::~Csock()
 {
 #ifdef ___DO_THREADS
-	m_cResolver.lock();
-	CDNSResolver::EStatus eStatus = m_cResolver.Status();
-	m_cResolver.unlock();
-	if ( eStatus == CDNSResolver::RUNNING )
-		m_cResolver.cancel();
+	if( m_pResolver )
+	{
+		m_pResolver->lock();
+		CDNSResolver::EStatus eStatus = m_pResolver->Status();
+		m_pResolver->unlock();
+		if ( eStatus == CDNSResolver::RUNNING )
+			m_pResolver->cancel();
+		Zzap( m_pResolver );
+	}
 #endif /* __DO_THREADS_ */
 
 	if ( m_iReadSock != m_iWriteSock )
 	{
-		CS_CLOSE( m_iReadSock );
-		CS_CLOSE( m_iWriteSock );
-	} else
+		if( m_iReadSock >= 0 )
+			CS_CLOSE( m_iReadSock );
+		if( m_iWriteSock >= 0 )
+			CS_CLOSE( m_iWriteSock );
+	} else if( m_iReadSock >= 0 )
 		CS_CLOSE( m_iReadSock );
 
 	m_iReadSock = -1;
@@ -418,6 +563,101 @@ Csock::~Csock()
 		CS_Delete( m_vcCrons[i] );
 }
 
+void Csock::Dereference()
+{
+#ifdef ___DO_THREADS
+	m_pResolver = NULL;
+#endif /* __DO_THREADS_ */
+	m_iWriteSock = m_iReadSock = -1;
+
+#ifdef HAVE_LIBSSL
+	m_ssl = NULL;
+	m_ssl_ctx = NULL;
+#endif /* HAVE_LIBSSL */
+
+	m_vcCrons.clear();
+}
+
+void Csock::Copy( const Csock & cCopy )
+{
+	m_iTcount		= cCopy.m_iTcount;
+	m_iLastCheckTimeoutTime	=	cCopy.m_iLastCheckTimeoutTime;
+	m_iport 		= cCopy.m_iport;
+   	m_iRemotePort	= cCopy.m_iRemotePort;
+	m_iLocalPort	= cCopy.m_iLocalPort;
+	m_iReadSock		= cCopy.m_iReadSock;
+	m_iWriteSock	= cCopy.m_iWriteSock;
+   	m_itimeout		= cCopy.m_iWriteSock;
+   	m_iConnType		= cCopy.m_iConnType;
+	m_iMethod		= cCopy.m_iMethod;
+	m_bssl			= cCopy.m_bssl;
+	m_bIsConnected	= cCopy.m_bIsConnected;
+	m_bBLOCK		= cCopy.m_bBLOCK;
+	m_bFullsslAccept	= cCopy.m_bFullsslAccept;
+	m_bsslEstablished	= cCopy.m_bsslEstablished;
+   	m_bEnableReadLine	= cCopy.m_bEnableReadLine;
+   	m_bRequireClientCert = cCopy.m_bRequireClientCert;
+   	m_bPauseRead		= cCopy.m_bPauseRead;
+	m_shostname		= cCopy.m_shostname;
+   	m_sbuffer		= cCopy.m_sbuffer;
+   	m_sSockName		= cCopy.m_sSockName;
+   	m_sPemFile		= cCopy.m_sPemFile;
+   	m_sCipherType	= cCopy.m_sCipherType;
+   	m_sParentName	= cCopy.m_sParentName;
+	m_sSend			= cCopy.m_sSend;
+   	m_sSSLBuffer	= cCopy.m_sSSLBuffer;
+   	m_sPemPass		= cCopy.m_sPemPass;
+   	m_sLocalIP		= cCopy.m_sLocalIP;
+   	m_sRemoteIP		= cCopy.m_sRemoteIP;
+	m_eCloseType	= cCopy.m_eCloseType;
+
+	m_iMaxMilliSeconds	= cCopy.m_iMaxMilliSeconds;
+   	m_iLastSendTime		= cCopy.m_iLastSendTime;
+   	m_iBytesRead		= cCopy.m_iBytesRead;
+   	m_iBytesWritten		= cCopy.m_iBytesWritten;
+   	m_iStartTime		= cCopy.m_iStartTime;
+	m_iMaxBytes			= cCopy.m_iMaxBytes;
+   	m_iLastSend			= cCopy.m_iLastSend;
+   	m_iMaxStoredBufferLength	= cCopy.m_iMaxStoredBufferLength;
+   	m_iTimeoutType		= cCopy.m_iTimeoutType;
+
+	m_address			= cCopy.m_address;
+   	m_bindhost			= cCopy.m_bindhost;
+	m_bIsIPv6			= cCopy.m_bIsIPv6;
+
+#ifdef HAVE_LIBSSL
+	FREE_SSL();
+	FREE_CTX(); // be sure to remove anything that was already here
+	m_ssl				= cCopy.m_ssl;
+	m_ssl_ctx			= cCopy.m_ssl_ctx;
+	m_ssl_method		= cCopy.m_ssl_method;
+
+	m_pCerVerifyCB		= cCopy.m_pCerVerifyCB;
+
+#endif /* HAVE_LIBSSL */
+
+	if( m_vcCrons.size() )
+	{
+		for( u_long a = 0; a < m_vcCrons.size(); a++ )
+		{
+			CS_Delete( m_vcCrons[a] );
+		}
+		m_vcCrons.clear();
+	}
+	m_vcCrons			= cCopy.m_vcCrons;
+
+	m_eConState			= cCopy.m_eConState;
+	m_sBindHost			= cCopy.m_sBindHost;
+	m_iCurBindCount		= cCopy.m_iCurBindCount;
+   	m_iDNSTryCount		= cCopy.m_iDNSTryCount;
+
+#ifdef ___DO_THREADS
+	if( m_pResolver )
+		CS_Delete( m_pResolver );
+	m_pResolver			= cCopy.m_pResolver;
+#endif /* ___DO_THREADS */
+}
+
 Csock & Csock::operator<<( const CS_STRING & s )
 {
 	Write( s );
@@ -789,7 +1029,14 @@ bool Csock::SSLClientSetup()
 				return( false );
 			}
 			break;
-
+		case TLS1:
+			m_ssl_method = TLSv1_client_method();
+			if ( !m_ssl_method )
+			{
+				CS_DEBUG( "WARNING: MakeConnection .... TLSv1_client_method failed!" );
+				return( false );
+			}
+			break;
 		case SSL23:
 		default:
 			m_ssl_method = SSLv23_client_method();
@@ -833,6 +1080,7 @@ bool Csock::SSLClientSetup()
 	SSL_set_rfd( m_ssl, m_iReadSock );
 	SSL_set_wfd( m_ssl, m_iWriteSock );
 	SSL_set_verify( m_ssl, SSL_VERIFY_PEER, ( m_pCerVerifyCB ? m_pCerVerifyCB : CertVerifyCB ) );
+	SSL_set_ex_data( m_ssl, GetCsockClassIdx(), this );
 
 	SSLFinishSetup( m_ssl );
 	return( true );
@@ -869,6 +1117,15 @@ bool Csock::SSLServerSetup()
 			}
 			break;
 
+		case TLS1:
+			m_ssl_method = TLSv1_server_method();
+			if ( !m_ssl_method )
+			{
+				CS_DEBUG( "WARNING: MakeConnection .... TLSv1_server_method failed!" );
+				return( false );
+			}
+			break;
+
 		case SSL23:
 		default:
 			m_ssl_method = SSLv23_server_method();
@@ -928,7 +1185,10 @@ bool Csock::SSLServerSetup()
 	SSL_set_wfd( m_ssl, m_iWriteSock );
 	SSL_set_accept_state( m_ssl );
 	if ( m_bRequireClientCert )
+	{
 		SSL_set_verify( m_ssl, SSL_VERIFY_FAIL_IF_NO_PEER_CERT|SSL_VERIFY_PEER, ( m_pCerVerifyCB ? m_pCerVerifyCB : CertVerifyCB ) );
+		SSL_set_ex_data( m_ssl, GetCsockClassIdx(), this );
+	}
 
 	SSLFinishSetup( m_ssl );
 	return( true );
@@ -1296,7 +1556,7 @@ int & Csock::GetWSock() { return( m_iWriteSock ); }
 void Csock::SetWSock( int iSock ) { m_iWriteSock = iSock; }
 void Csock::SetSock( int iSock ) { m_iWriteSock = iSock; m_iReadSock = iSock; }
 int & Csock::GetSock() { return( m_iReadSock ); }
-void Csock::ResetTimer() { m_iTcount = 0; }
+void Csock::ResetTimer() { m_iLastCheckTimeoutTime = 0; m_iTcount = 0; }
 void Csock::PauseRead() { m_bPauseRead = true; }
 bool Csock::IsReadPaused() { return( m_bPauseRead ); }
 
@@ -1317,20 +1577,47 @@ void Csock::SetTimeoutType( u_int iTimeoutType ) { m_iTimeoutType = iTimeoutType
 int Csock::GetTimeout() const { return m_itimeout; }
 u_int Csock::GetTimeoutType() const { return( m_iTimeoutType ); }
 
-bool Csock::CheckTimeout()
+bool Csock::CheckTimeout( time_t iNow )
 {
+	if( m_iLastCheckTimeoutTime == 0 )
+	{
+		m_iLastCheckTimeoutTime = iNow;
+		return( false );
+	}
+
 	if ( IsReadPaused() )
 		return( false );
 
+	time_t iDiff = 0;
+	if( iNow > m_iLastCheckTimeoutTime )
+		iDiff = iNow - m_iLastCheckTimeoutTime;
+	else
+	{
+		// this is weird, but its possible if someone changes a clock and it went back in time, this essentially has to reset the last check
+		// the worst case scenario is the timeout is about to it and the clock changes, it would then cause
+		// this to pass over the last half the time
+		m_iLastCheckTimeoutTime = iNow; 
+	}
+
 	if ( m_itimeout > 0 )
 	{
-		if ( m_iTcount >= m_itimeout )
+		// this is basically to help stop a clock adjust ahead, stuff could reset immediatly on a clock jump
+		// otherwise
+		time_t iRealTimeout = m_itimeout;
+		if( iRealTimeout <= 1 )
+			m_iTcount++;
+		else if( m_iTcount == 0 )
+			iRealTimeout /= 2;
+		if( iDiff >= iRealTimeout )
 		{
-			Timeout();
-			return( true );
+			if( m_iTcount == 0 )
+				m_iLastCheckTimeoutTime = iNow - iRealTimeout;
+			if( m_iTcount++ >= 1 )
+			{
+				Timeout();
+				return( true );
+			}
 		}
-
-		m_iTcount++;
 	}
 	return( false );
 }
@@ -1516,6 +1803,13 @@ int Csock::PemPassCB( char *buf, int size, int rwflag, void *pcSocket )
 
 int Csock::CertVerifyCB( int preverify_ok, X509_STORE_CTX *x509_ctx )
 {
+	/*
+	 * A small quick example on how to get ahold of the Csock in the data portion of x509_ctx
+	Csock *pSock = GetCsockFromCTX( x509_ctx );
+	assert( pSock );
+	cerr << pSock->GetRemoteIP() << endl;
+	 */
+
 	/* return 1 always for now, probably want to add some code for cert verification */
 	return( 1 );
 }
@@ -1693,6 +1987,8 @@ void Csock::DelCron( const CS_STRING & sName, bool bDeleteAll, bool bCaseSensiti
 			m_vcCrons[a]->Stop();
 			CS_Delete( m_vcCrons[a] );
 			m_vcCrons.erase( m_vcCrons.begin() + a-- );
+			if( !bDeleteAll )
+				break;
 		}
 	}
 }
@@ -1758,44 +2054,47 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 	}
 
 #ifdef ___DO_THREADS
+	if( !m_pResolver )
+		m_pResolver = new CDNSResolver;
+
 	if ( m_iDNSTryCount == 0 )
 	{
-		m_cResolver.Lookup( ( eDNSLType == DNS_VHOST ) ? m_sBindHost : m_shostname );
+		m_pResolverLookup( ( eDNSLType == DNS_VHOST ) ? m_sBindHost : m_shostname );
 		m_iDNSTryCount++;
 	}
    
-	if ( m_cResolver.IsCompleted() )
+	if ( m_pResolverIsCompleted() )
 	{
 		m_iDNSTryCount = 0;
-		if ( m_cResolver.Suceeded() )
+		if ( m_pResolverSuceeded() )
 		{
 			if ( eDNSLType == DNS_VHOST )
 			{
-				if( !m_cResolver.GetSockAddr()->GetIPv6() )
+				if( !m_pResolverGetSockAddr()->GetIPv6() )
 				{
 					SetIPv6( false );
-					memcpy( m_bindhost.GetAddr(), m_cResolver.GetSockAddr()->GetAddr(), sizeof( *(m_bindhost.GetAddr()) ) );
+					memcpy( m_bindhost.GetAddr(), m_pResolverGetSockAddr()->GetAddr(), sizeof( *(m_bindhost.GetAddr()) ) );
 				}
 #ifdef HAVE_IPV6
 				else
 				{
 					SetIPv6( true );
-					memcpy( m_bindhost.GetAddr6(), m_cResolver.GetSockAddr()->GetAddr6(), sizeof( *(m_bindhost.GetAddr6()) ) );
+					memcpy( m_bindhost.GetAddr6(), m_pResolverGetSockAddr()->GetAddr6(), sizeof( *(m_bindhost.GetAddr6()) ) );
 				}
 #endif /* HAVE_IPV6 */
 			}
 			else
 			{
-				if( m_cResolver.GetSockAddr()->GetIPv6() )
+				if( m_pResolverGetSockAddr()->GetIPv6() )
 				{
 					SetIPv6( false );
-					memcpy( m_address.GetAddr(), m_cResolver.GetSockAddr()->GetAddr(), sizeof( *(m_address.GetAddr()) ) );
+					memcpy( m_address.GetAddr(), m_pResolverGetSockAddr()->GetAddr(), sizeof( *(m_address.GetAddr()) ) );
 				}
 #ifdef HAVE_IPV6
 				else
 				{
 					SetIPv6( true );
-					memcpy( m_address.GetAddr6(), m_cResolver.GetSockAddr()->GetAddr6(), sizeof( *(m_address.GetAddr6()) ) );
+					memcpy( m_address.GetAddr6(), m_pResolverGetSockAddr()->GetAddr6(), sizeof( *(m_address.GetAddr6()) ) );
 				}
 #endif /* HAVE_IPV6 */
 			}
@@ -1931,6 +2230,7 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 	m_ssl = NULL;
 	m_ssl_ctx = NULL;
 #endif /* HAVE_LIBSSL */
+	m_iTcount = 0;
 	m_iReadSock = -1;
 	m_iWriteSock = -1;
 	m_itimeout = itimeout;
@@ -1938,7 +2238,6 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 	m_bIsConnected = false;
 	m_iport = iport;
 	m_shostname = sHostname;
-	m_iTcount = 0;
 	m_sbuffer.clear();
 	m_eCloseType = CLT_DONT;
 	m_bBLOCK = true;
@@ -1965,5 +2264,6 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 	m_iDNSTryCount = 0;
 	m_iCurBindCount = 0;
 	m_bIsIPv6 = false;
+	m_iLastCheckTimeoutTime = 0;
 }
 
