@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.27 $
+* $Revision: 1.178 $
 */
 
 // note to compile with win32 need to link to winsock2, using gcc its -lws2_32
@@ -102,7 +102,7 @@
 #endif /* CS_DEBUG */
 
 #ifdef __DEBUG__
-#	define PERROR( f ) __Perror( f )
+#	define PERROR( f ) __Perror( f, __FILE__, __LINE__ )
 #else
 #	define PERROR( f )	(void)0
 #endif /* __DEBUG__ */
@@ -112,6 +112,27 @@ namespace Csocket
 {
 #endif /* _NO_CSOCKET_NS */
 
+/**
+ * @class CSCharBuffer
+ * @brief ease of use self deleting char * class
+ */
+class CSCharBuffer
+{
+public:
+	CSCharBuffer( size_t iSize )
+	{
+		m_pBuffer = (char *)malloc( iSize );
+	}
+	~CSCharBuffer()
+	{
+		free( m_pBuffer );
+	}
+	char * operator()() { return( m_pBuffer ); }
+
+private:
+	char *	m_pBuffer;
+};
+
 class CSSockAddr
 {
 public:
@@ -199,6 +220,15 @@ private:
 class Csock;
 int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockAddr );
 
+//! used to retrieve the context position of the socket to its associated ssl connection. Setup once in InitSSL() via SSL_get_ex_new_index
+int GetCsockClassIdx();
+
+#ifdef HAVE_LIBSSL
+//! returns the sock object associated to the particular context. returns NULL on failure or if not available
+Csock *GetCsockFromCTX( X509_STORE_CTX *pCTX );
+#endif /* HAVE_LIBSSL */
+
+
 #if defined( _REENTRANT ) && defined( _USE_THREADED_DNS )
 #define ___DO_THREADS
 #include <pthread.h>
@@ -347,7 +377,7 @@ inline void TFD_CLR( u_int iSock, fd_set *set )
 	FD_CLR( iSock, set );
 }
 
-void __Perror( const CS_STRING & s );
+void __Perror( const CS_STRING & s, const char *pszFile, unsigned int iLineNo );
 unsigned long long millitime();
 
 
@@ -396,6 +426,9 @@ public:
 	const CS_STRING & GetName() const;
 	void SetName( const CS_STRING & sName );
 
+	//! returns the timestamp of the next estimated run time. Note that it may not run at this EXACT time, but it will run at least at this time or after
+	time_t GetNextRun() const { return( m_iTime ); }
+
 public:
 
 	//! this is the method you should override
@@ -441,6 +474,16 @@ public:
 
 	virtual ~Csock();
 
+	/**
+	 * @brief in the event you pass this class to Copy(), you MUST call this function or
+	 * on the original Csock other wise bad side effects will happen (double deletes, weird sock closures, etc)
+	 * if you call this function and have not handled the internal pointers, other bad things can happend (memory leaks, fd leaks, etc)
+	 * the whole point of this function is to allow this class to go away without shutting down 
+	 */
+	virtual void Dereference();
+	//! use this to copy a sock from one to the other, override it if you have special needs in the event of a copy
+	virtual void Copy( const Csock & cCopy );
+	
 	enum ETConn
 	{
 		OUTBOUND			= 0,		//!< outbound connection
@@ -469,7 +512,8 @@ public:
 	{
 		SSL23				= 0,
 		SSL2				= 2,
-		SSL3				= 3
+		SSL3				= 3,
+		TLS1				= 4
 	};
 
 	enum ECONState
@@ -631,7 +675,7 @@ public:
 	u_int GetTimeoutType() const;
 
 	//! returns true if the socket has timed out
-	virtual bool CheckTimeout();
+	virtual bool CheckTimeout( time_t iNow );
 
 	/**
 	* pushes data up on the buffer, if a line is ready
@@ -774,7 +818,12 @@ public:
 
 	//! insert a newly created cron
 	virtual void AddCron( CCron * pcCron );
-	//! delete cron(s) by name
+	/**
+	 * @brief deletes a cron by name
+	 * @param sName the name of the cron
+	 * @param bDeleteAll delete all crons that match sName
+	 * @param bCaseSensitive use strcmp or strcasecmp
+	 */
 	virtual void DelCron( const CS_STRING & sName, bool bDeleteAll = true, bool bCaseSensitive = true );
 	//! delete cron by idx
 	virtual void DelCron( u_int iPos );
@@ -880,8 +929,14 @@ public:
 #endif /* HAVE_LIBSSL */
 
 
-	//! return how long it has been (in seconds) since the last write
-	int GetTimeSinceLastWrite() { return m_iTcount; }
+	//! return how long it has been (in seconds) since the last read or successful write
+	int GetTimeSinceLastDataTransaction( time_t iNow = 0 ) 
+	{ 
+		if( m_iLastCheckTimeoutTime == 0 )
+			return( 0 );
+		return( ( iNow > 0 ? iNow : time( NULL ) ) - m_iLastCheckTimeoutTime ); 
+	}
+	time_t GetLastCheckTimeout() { return( m_iLastCheckTimeoutTime ); }
 
 	//! return the data imediatly ready for read
 	virtual int GetPending();
@@ -944,9 +999,16 @@ public:
 	//! returns true if this socket can write its data, primarily used with rate shaping, initialize iNOW to 0 and it sets it on the first call
 	bool AllowWrite( unsigned long long & iNOW ) const;
 
+	//! returns a const reference to the crons associated to this socket
+	const std::vector<CCron *> & GetCrons() const { return( m_vcCrons ); }
+
 private:
+	//! making private for safety
+	Csock( const Csock & cCopy ) {}
+
+	// NOTE! if you add any new members, be sure to add them to Copy()
 	u_short		m_iport, m_iRemotePort, m_iLocalPort;
-	int			m_iReadSock, m_iWriteSock, m_itimeout, m_iConnType, m_iTcount, m_iMethod;
+	int			m_iReadSock, m_iWriteSock, m_itimeout, m_iConnType, m_iMethod, m_iTcount;
 	bool		m_bssl, m_bIsConnected, m_bBLOCK, m_bFullsslAccept;
 	bool		m_bsslEstablished, m_bEnableReadLine, m_bRequireClientCert, m_bPauseRead;
 	CS_STRING	m_shostname, m_sbuffer, m_sSockName, m_sPemFile, m_sCipherType, m_sParentName;
@@ -958,6 +1020,7 @@ private:
 
 	CSSockAddr 		m_address, m_bindhost;
 	bool			m_bIsIPv6;
+	time_t			m_iLastCheckTimeoutTime;
 
 #ifdef HAVE_LIBSSL
 	SSL 				*m_ssl;
@@ -966,16 +1029,16 @@ private:
 
 	FPCertVerifyCB		m_pCerVerifyCB;
 
-	virtual void FREE_SSL();
-	virtual void FREE_CTX();
+	void FREE_SSL();
+	void FREE_CTX();
 
 #endif /* HAVE_LIBSSL */
 
 	std::vector<CCron *>		m_vcCrons;
 
 	//! Create the socket
-	virtual int SOCKET( bool bListen = false );
-	virtual void Init( const CS_STRING & sHostname, u_short iport, int itimeout = 60 );
+	int SOCKET( bool bListen = false );
+	void Init( const CS_STRING & sHostname, u_short iport, int itimeout = 60 );
 
 
 	// Connection State Info
@@ -984,7 +1047,7 @@ private:
 	u_int			m_iCurBindCount, m_iDNSTryCount;
 
 #ifdef ___DO_THREADS
-	CDNSResolver	m_cResolver;
+	CDNSResolver	m_pResolver;
 #endif /* ___DO_THREADS */
 
 };
@@ -1100,7 +1163,7 @@ public:
 	}
 	virtual ~CSListener() {}
 
-	const u_short GetPort() const { return( m_iPort ); }
+	u_short GetPort() const { return( m_iPort ); }
 	const CS_STRING & GetSockName() const { return( m_sSockName ); }
 	const CS_STRING & GetBindHost() const { return( m_sBindHost ); }
 	bool GetIsSSL() const { return( m_bIsSSL ); }
@@ -1329,12 +1392,13 @@ public:
 		return( false );
 	}
 
+
 	/**
 	* Best place to call this class for running, all the call backs are called.
 	* You should through this in your main while loop (long as its not blocking)
 	* all the events are called as needed.
 	*/
-	virtual void Loop ()
+	virtual void Loop()
 	{
 		for( u_int a = 0; a < this->size(); a++ )
 		{
@@ -1406,7 +1470,6 @@ public:
 
 		std::map<T *, EMessages> mpeSocks;
 		Select( mpeSocks );
-		std::set<T *> spReadySocks;
 
 		switch( m_errno )
 		{
@@ -1421,22 +1484,17 @@ public:
 					{
 						// read in data
 						// if this is a
-						char *buff;
 						int iLen = 0;
 
 						if ( pcSock->GetSSL() )
 							iLen = pcSock->GetPending();
 
-						if ( iLen > 0 )
-						{
-							buff = (char *)malloc( iLen );
-						} else
-						{
+						if ( iLen <= 0 )
 							iLen = CS_BLOCKSIZE;
-							buff = (char *)malloc( CS_BLOCKSIZE );
-						}
+						
+						CSCharBuffer cBuff( iLen );
 
-						int bytes = pcSock->Read( buff, iLen );
+						int bytes = pcSock->Read( cBuff(), iLen );
 
 						if ( ( bytes != T::READ_TIMEDOUT ) && ( bytes != T::READ_CONNREFUSED )
 							&& ( !pcSock->IsConnected() ) )
@@ -1478,14 +1536,12 @@ public:
 								if ( T::TMO_READ & pcSock->GetTimeoutType() )
 									pcSock->ResetTimer();	// reset the timeout timer
 
-								pcSock->ReadData( buff, bytes );	// Call ReadData() before PushBuff() so that it is called before the ReadLine() event - LD  07/18/05
-								pcSock->PushBuff( buff, bytes );
+								pcSock->ReadData( cBuff(), bytes );	// Call ReadData() before PushBuff() so that it is called before the ReadLine() event - LD  07/18/05
+								pcSock->PushBuff( cBuff(), bytes );
 								break;
 							}
 						}
 
-						// free up the buff
-						free( buff );
 					} else if ( iErrno == SELECT_ERROR )
 					{
 						// a socket came back with an error
@@ -1503,7 +1559,7 @@ public:
 		}
 
 		unsigned long long iMilliNow = millitime();
-		if ( ( iMilliNow - m_iCallTimeouts ) > 1000 )
+		if ( ( iMilliNow - m_iCallTimeouts ) >= 1000 )
 		{
 			m_iCallTimeouts = iMilliNow;
 			// call timeout on all the sockets that recieved no data
@@ -1512,7 +1568,7 @@ public:
 				if ( (*this)[i]->GetConState() != T::CST_OK )
 					continue;
 
-				if ( (*this)[i]->CheckTimeout() )
+				if ( (*this)[i]->CheckTimeout( iMilliNow / 1000 ) )
 					DelSock( i-- );
 			}
 		}
@@ -1520,6 +1576,37 @@ public:
 		Cron();
 	}
 
+	/**
+	 * @brief this is similar to loop, except that it dynamically adjusts the select time based on jobs and timeouts in sockets
+	 *
+	 *	- This type of behavior only works well in a scenario where there is low traffic. If you use this then its because you
+	 *	- are trying to spare yourself some of those idle loops where nothing is done. If you try to use this code where you have lots of
+	 *	- connections and/or lots of traffic you might end up causing more CPU usage than just a plain Loop() with a static sleep of 500ms
+	 *	- its a trade off at some point, and you'll probably find out that the vast majority of the time and in most cases Loop() works fine
+	 *	- by itself. I've tried to mitigate that as much as possible by not having it change the select if the previous call to select
+	 *	- was not a timeout. Anyways .... Caveat Emptor.
+	 *	- Sample useage is cFoo.DynamicSelectLoop( 500000, 5000000 ); which basically says min of 500ms and max of 5s
+	 * 
+	 * @param iLowerBounds the lower bounds to use in MICROSECONDS
+	 * @param iUpperBounds the upper bounds to use in MICROSECONDS
+	 * @param iMaxResolution the maximum time to calculate overall in seconds
+	 */
+	void DynamicSelectLoop( u_long iLowerBounds, u_long iUpperBounds, time_t iMaxResolution = 3600 )
+	{
+		SetSelectTimeout( iLowerBounds );	
+		if( m_errno == SELECT_TIMEOUT )
+		{ // only do this if the previous call to select was a timeout
+			time_t iNow = time( NULL );
+			u_long iSelectTimeout = GetDynamicSleepTime( iNow, iMaxResolution );
+			iSelectTimeout *= 1000000;
+			iSelectTimeout = std::max( iLowerBounds, iSelectTimeout );
+			iSelectTimeout = std::min( iSelectTimeout, iUpperBounds );
+			if( iLowerBounds != iSelectTimeout )
+				SetSelectTimeout( iSelectTimeout );
+		}
+		Loop();
+	}
+
 	/**
 	* Make this method virtual, so you can override it when a socket is added.
 	* Assuming you might want to do some extra stuff
@@ -1604,7 +1691,12 @@ public:
 		m_vcCrons.push_back( pcCron );
 	}
 
-	//! delete cron(s) by name
+	/**
+	 * @brief deletes a cron by name
+	 * @param sName the name of the cron
+	 * @param bDeleteAll delete all crons that match sName
+	 * @param bCaseSensitive use strcmp or strcasecmp
+	 */
 	virtual void DelCron( const CS_STRING & sName, bool bDeleteAll = true, bool bCaseSensitive = true )
 	{
 		for( u_int a = 0; a < m_vcCrons.size(); a++ )
@@ -1615,6 +1707,8 @@ public:
 				m_vcCrons[a]->Stop();
 				CS_Delete( m_vcCrons[a] );
 				m_vcCrons.erase( m_vcCrons.begin() + a-- );
+				if( !bDeleteAll )
+					break;
 			}
 		}
 	}
@@ -1645,10 +1739,10 @@ public:
 	}
 
 	//! Get the Select Timeout in MICROSECONDS ( 1000 == 1 millisecond )
-	u_int GetSelectTimeout() { return( m_iSelectWait ); }
+	u_long GetSelectTimeout() { return( m_iSelectWait ); }
 	//! Set the Select Timeout in MICROSECODS ( 1000 == 1 millisecond )
 	//! Setting this to 0 will cause no timeout to happen, Select() will return instantly
-	void  SetSelectTimeout( u_int iTimeout ) { m_iSelectWait = iTimeout; }
+	void  SetSelectTimeout( u_long iTimeout ) { m_iSelectWait = iTimeout; }
 
 	std::vector<CCron *> & GetCrons() { return( m_vcCrons ); }
 
@@ -1680,7 +1774,7 @@ public:
 			return;
 		}
 
-		Csock * pSock = (*this)[iPos];
+		T * pSock = (*this)[iPos];
 
 		if ( pSock->IsConnected() )
 			pSock->Disconnected(); // only call disconnected event if connected event was called (IE IsConnected was set)
@@ -1692,6 +1786,44 @@ public:
 		this->erase( this->begin() + iPos );
 	}
 
+	/**
+	 * @brief swaps out a sock with a copy of the original sock
+	 * @param pNewSock the new sock to change out with. (this should be constructed by you with the default ctor)
+	 * @param iOrginalSockIdx the position in this sockmanager of the original sock
+	 * @return true on success
+	 */
+	virtual bool SwapSockByIdx( Csock *pNewSock, u_long iOrginalSockIdx )
+	{
+		if( iOrginalSockIdx >= this->size() )
+		{
+			CS_DEBUG( "Invalid Sock Position Requested! [" << iOrginalSockIdx << "]" );
+			return( false );
+		}
+
+		Csock *pSock = (*this)[iOrginalSockIdx];
+		pNewSock->Copy( *pSock );
+		pSock->Dereference();
+		CS_Delete( pSock );
+		(*this)[iOrginalSockIdx] = (T *)pNewSock;
+		return( true );
+	}
+	
+	/**
+	 * @brief swaps out a sock with a copy of the original sock
+	 * @param pNewSock the new sock to change out with. (this should be constructed by you with the default ctor)
+	 * @param pOrigSock the address of the original socket
+	 * @return true on success
+	 */
+	virtual bool SwapSockByAddr( Csock *pNewSock, Csock *pOrigSock )
+	{
+		for( u_long a = 0; a < this->size(); a++ )
+		{
+			if( (*this)[a] == pOrigSock )
+				return( SwapSockByIdx( pNewSock, a ) );
+		}
+		return( false );
+	}
+
 	//! Get the bytes read from all sockets current and past
 	unsigned long long GetBytesRead() const
 	{
@@ -1731,9 +1863,8 @@ private:
 		struct timeval tv;
 		fd_set rfds, wfds;
 
-		tv.tv_sec = 0;
-		tv.tv_usec = m_iSelectWait;
-
+		tv.tv_sec = m_iSelectWait / 1000000;
+		tv.tv_usec = m_iSelectWait % 1000000;
 		u_int iQuickReset = 100;
 		if ( m_iSelectWait == 0 )
 			iQuickReset = 0;
@@ -1782,6 +1913,7 @@ private:
 				if ( ( pcSock->GetSSL() ) && ( pcSock->GetType() == T::INBOUND ) && ( !pcSock->FullSSLAccept() ) )
 				{
 					tv.tv_usec = iQuickReset;	// just make sure this returns quick incase we still need pending
+					tv.tv_sec = 0;
 					// try accept on this socket again
 					if ( !pcSock->AcceptSSL() )
 						pcSock->Close();
@@ -1837,16 +1969,21 @@ private:
 		int iSel;
 
 		if ( !mpeSocks.empty() ) // .1 ms pause to see if anything else is ready (IE if there is SSL data pending, don't wait too long)
+		{
 			tv.tv_usec = iQuickReset;
+			tv.tv_sec = 0;
+		}
 		else if ( ( !this->empty() ) && ( !bHasAvailSocks ) )
+		{
 			tv.tv_usec = iQuickReset;
+			tv.tv_sec = 0;
+		}
 
 
 		if ( bHasWriteable )
 			iSel = select(FD_SETSIZE, &rfds, &wfds, NULL, &tv);
 		else
 			iSel = select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
-
 		if ( iSel == 0 )
 		{
 			if ( mpeSocks.empty() )
@@ -1983,6 +2120,39 @@ private:
 		}
 	}
 
+	time_t GetDynamicSleepTime( time_t iNow, time_t iMaxResolution = 3600 ) const
+	{
+		time_t iNextRunTime = iNow + iMaxResolution;
+		time_t iMinTimeout = iMaxResolution;
+		for( u_long a = 0; a < this->size() && iMinTimeout; a++ )
+		{
+			if( (*this)[a]->GetConState() != T::CST_OK )
+				iMinTimeout = 0; // this is in a nebulous state, need to let it proceed like normal
+
+			time_t iTimeoutInSeconds = (*this)[a]->GetTimeout();
+			if( iTimeoutInSeconds > 0 )
+			{
+				time_t iLastTimeData = (*this)[a]->GetLastCheckTimeout();
+				time_t iDiff = iNow - iLastTimeData;
+				if( iDiff > iTimeoutInSeconds )
+					iTimeoutInSeconds = 0;
+				else
+					iTimeoutInSeconds -= iDiff;
+			
+				iMinTimeout = std::min( iMinTimeout, iTimeoutInSeconds );
+			}
+
+			const std::vector<CCron *> & vCrons = (*this)[a]->GetCrons();
+			for( u_long b = 0; b < vCrons.size(); b++ )
+				iNextRunTime = std::min( iNextRunTime, vCrons[b]->GetNextRun() );
+		}
+		for( u_long a = 0; a < m_vcCrons.size(); a++ )
+			iNextRunTime = std::min( iNextRunTime, m_vcCrons[a]->GetNextRun() );
+
+		if( iNextRunTime < iNow )
+			return( 0 ); // smallest unit possible
+		return( std::min( iNextRunTime - iNow, iMinTimeout ) );
+	}
 
 	//! internal use only
 	virtual void SelectSock( std::map<T *, EMessages> & mpeSocks, EMessages eErrno, T * pcSock )
@@ -2023,7 +2193,7 @@ private:
 	unsigned long long		m_iCallTimeouts;
 	unsigned long long		m_iBytesRead;
 	unsigned long long		m_iBytesWritten;
-	u_int					m_iSelectWait;
+	u_long					m_iSelectWait;
 };
 
 //! basic socket class
