@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "HTTPSock.h"
 #include "znc.h"
@@ -325,7 +331,7 @@ bool CHTTPSock::PrintHeader(unsigned long uContentLength, const CString& sConten
 
 	Write("HTTP/1.0 " + CString(uStatusId) + " " + sStatusMsg + "\r\n");
 	//Write("Date: Tue, 28 Jun 2005 20:45:36 GMT\r\n");
-	Write("Server: ZNC " + CZNC::GetTag() + "\r\n");
+	Write("Server: " + CZNC::GetTag() + "\r\n");
 	Write("Content-Length: " + CString(uContentLength) + "\r\n");
 	Write("Connection: Close\r\n");
 	Write("Content-Type: " + m_sContentType + "\r\n");
