@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "znc.h"
 #include "IRCSock.h"
@@ -49,8 +55,15 @@ CIRCSock::~CIRCSock() {
 		CZNC::Get().ReleaseISpoof();
 	}
 
-	PutIRC("QUIT :" + m_pUser->GetQuitMsg());
+	Quit();
 	m_msChans.clear();
+	GetUser()->AddBytesRead(GetBytesRead());
+	GetUser()->AddBytesWritten(GetBytesWritten());
+}
+
+void CIRCSock::Quit() {
+	PutIRC("QUIT :" + m_pUser->GetQuitMsg());
+	Close(CLT_AFTERWRITE);
 }
 
 void CIRCSock::ReadLine(const CString& sData) {
@@ -92,10 +105,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 					SetTimeout(900);	// Now that we are connected, let nature take its course
 					PutIRC("WHO " + sNick);
 
-					SetNick(sNick);
-
-					MODULECALL(OnIRCConnected(), m_pUser, NULL, );
-
 					m_bAuthed = true;
 					m_pUser->PutStatus("Connected!");
 
@@ -103,7 +112,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 					for (unsigned int a = 0; a < vClients.size(); a++) {
 						CClient* pClient = vClients[a];
-						CString sClientNick = pClient->GetNick();
+						CString sClientNick = pClient->GetNick(false);
 
 						if (sClientNick.CaseCmp(sNick) != 0) {
 							// If they connected with a nick that doesn't match the one we got on irc, then we need to update them
@@ -111,16 +120,13 @@ void CIRCSock::ReadLine(const CString& sData) {
 						}
 					}
 
+					SetNick(sNick);
+
+					MODULECALL(OnIRCConnected(), m_pUser, NULL, );
+
 					m_pUser->ClearRawBuffer();
 					m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 
-					// Now that we are connected, we need to join our chans
-					const vector<CChan*>& vChans = m_pUser->GetChans();
-
-					for (unsigned int a = 0; a < vChans.size(); a++) {
-						PutIRC("JOIN " + vChans[a]->GetName() + " " + vChans[a]->GetKey());
-					}
-
 					CZNC::Get().ReleaseISpoof();
 					m_bISpoofReleased = true;
 
@@ -128,6 +134,8 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 				case 5:
 					ParseISupport(sRest);
+					m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+					break;
 				case 2:
 				case 3:
 				case 4:
@@ -138,10 +146,10 @@ void CIRCSock::ReadLine(const CString& sData) {
 				case 255:	// client count
 				case 265:	// local users
 				case 266:	// global users
-					m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
+					m_pUser->UpdateRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 					break;
 				case 422:	// MOTD File is missing
-				case 375:	// begin motd
+				case 375: 	// begin motd
 					m_pUser->ClearMotdBuffer();
 				case 372:	// motd
 				case 376:	// end motd
@@ -161,6 +169,12 @@ void CIRCSock::ReadLine(const CString& sData) {
 					}
 
 					break;
+				case 437:
+					// :irc.server.net 437 * badnick :Nick/channel is temporarily unavailable
+					// :irc.server.net 437 mynick badnick :Nick/channel is temporarily unavailable
+					// :irc.server.net 437 mynick badnick :Cannot change nickname while banned on channel
+					if (m_pUser->IsChan(sRest.Token(0)) || sNick != "*")
+						break;
 				case 433: {
 					unsigned int uMax = GetMaxNickLen();
 					CString sBadNick = sRest.Token(0);
@@ -186,14 +200,14 @@ void CIRCSock::ReadLine(const CString& sData) {
 						} else {
 							char cLetter = 0;
 							if (sBadNick.empty()) {
-								Close();
+								Quit();
 								return;
 							}
 
 							cLetter = sBadNick.Right(1)[0];
 
 							if (cLetter == 'z') {
-								Close();
+								Quit();
 								return;
 							}
 
@@ -325,39 +339,46 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 					pChan->AddNicks(sNicks);
 
-					CString sTmp;
+					// Get everything except the actual user list
+					CString sTmp = sLine.Token(0, false, " :") + " :";
 					vector<CClient*>& vClients = m_pUser->GetClients();
+					CString sNick;
+
 					for (unsigned int a = 0; a < vClients.size(); a++) {
-						if((!m_bNamesx || vClients[a]->HasNamesx()) &&
-							(!m_bUHNames || vClients[a]->HasUHNames())) {
+						if ((!m_bNamesx || vClients[a]->HasNamesx())
+								&& (!m_bUHNames || vClients[a]->HasUHNames())) {
 							m_pUser->PutUser(sLine, vClients[a]);
 						} else {
-							sTmp = sLine.Token(0) + " ";
-							sTmp += sLine.Token(1) + " ";
-							sTmp += sLine.Token(2) + " ";
-							sTmp += sLine.Token(3) + " ";
-							sTmp += sLine.Token(4) + " :";
-							while(!sNicks.empty()) {
-								CString sNick = sNicks.Token(0);
-								sNicks = sNicks.Token(1, true);
-								if(m_bNamesx && !vClients[a]->HasNamesx()
+							unsigned int i = 0;
+							// This loop runs once more than there are nicks,
+							// the last run sets sNick to "" and causes the break.
+							do {
+								sNick = sNicks.Token(i);
+								if (m_bNamesx && !vClients[a]->HasNamesx()
 									&& IsPermChar(sNick[0])) {
-									while(sNick.length() > 2
-										&& IsPermChar(sNick[1])) {
+									// Server has, client hasnt NAMESX,
+									// so we just use the first perm char
+									while (sNick.length() > 2
+											&& IsPermChar(sNick[1])) {
 										sNick = sNick[0] + sNick.substr(2);
 									}
 								}
 
-								if(m_bUHNames && !vClients[a]->HasUHNames()) {
+								// Server has, client hasnt UHNAMES,
+								// so we strip away ident and host.
+								if (m_bUHNames && !vClients[a]->HasUHNames()) {
 									sNick = sNick.Token(0, false, "!");
 								}
 
 								sTmp += sNick + " ";
-							}
-							sTmp.RightChomp();
+								i++;
+							} while (!sNick.empty());
+							// Strip away the spaces we inserted at the end
+							sTmp.Trim(" ");
 							m_pUser->PutUser(sTmp, vClients[a]);
 						}
 					}
+					// We forwarded it in the for loop above already
 					return;
 				}
 				case 366: {	// end of names list
@@ -473,12 +494,20 @@ void CIRCSock::ReadLine(const CString& sData) {
 					sChan.LeftChomp();
 				}
 
+				CChan* pChan;
+
 				// Todo: use nick compare function
 				if (Nick.GetNick().CaseCmp(GetNick()) == 0) {
 					m_pUser->AddChan(sChan, false);
+					pChan = m_pUser->FindChan(sChan);
+					if (pChan) {
+						pChan->ResetJoinTries();
+						pChan->Enable();
+					}
+				} else {
+					pChan = m_pUser->FindChan(sChan);
 				}
 
-				CChan* pChan = m_pUser->FindChan(sChan);
 				if (pChan) {
 					pChan->AddNick(Nick.GetNickMask());
 					MODULECALL(OnJoin(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
@@ -494,9 +523,13 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 
 				CChan* pChan = m_pUser->FindChan(sChan);
+				bool bDetached = false;
 				if (pChan) {
 					pChan->RemNick(Nick.GetNick());
 					MODULECALL(OnPart(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
+
+					if (pChan->IsDetached())
+						bDetached = true;
 				}
 
 				// Todo: use nick compare function
@@ -504,20 +537,52 @@ void CIRCSock::ReadLine(const CString& sData) {
 					m_pUser->DelChan(sChan);
 				}
 
-				if ((pChan) && (pChan->IsDetached())) {
+				/*
+				 * We use this boolean because
+				 * m_pUser->DelChan() will delete this channel
+				 * and thus we would dereference an
+				 * already-freed pointer!
+				 */
+				if (bDetached) {
 					return;
 				}
 			} else if (sCmd.CaseCmp("MODE") == 0) {
-				CString sChan = sRest.Token(0);
+				CString sTarget = sRest.Token(0);
 				CString sModes = sRest.Token(1, true);
+				if(sModes.Left(1) == ":")
+					sModes = sModes.substr(1);
 
-				CChan* pChan = m_pUser->FindChan(sChan);
+				CChan* pChan = m_pUser->FindChan(sTarget);
 				if (pChan) {
 					pChan->ModeChange(sModes, Nick.GetNick());
 
 					if (pChan->IsDetached()) {
 						return;
 					}
+				} else if (sTarget == m_Nick.GetNick()) {
+					CString sModeArg = sModes.Token(0);
+//					CString sArgs = sModes.Token(1, true); Usermode changes got no params
+					bool bAdd = true;
+/* no module call defined (yet?)
+#ifdef _MODULES
+					MODULECALL(OnRawUserMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, );
+#endif
+*/
+					for (unsigned int a = 0; a < sModeArg.size(); a++) {
+						const unsigned char& uMode = sModeArg[a];
+
+						if (uMode == '+') {
+							bAdd = true;
+						} else if (uMode == '-') {
+							bAdd = false;
+						} else {
+							if(bAdd) {
+								m_scUserModes.insert(uMode);
+							} else {
+								m_scUserModes.erase(uMode);
+							}
+						}
+					}
 				}
 			} else if (sCmd.CaseCmp("KICK") == 0) {
 				// :opnick!ident@host.com KICK #chan nick :msg
@@ -533,15 +598,11 @@ void CIRCSock::ReadLine(const CString& sData) {
 					MODULECALL(OnKick(Nick.GetNickMask(), sKickedNick, *pChan, sMsg), m_pUser, NULL, );
 				}
 
-				if (GetNick().CaseCmp(sKickedNick) == 0) {
-					CString sKey;
+				if (GetNick().CaseCmp(sKickedNick) == 0 && pChan) {
+					pChan->SetIsOn(false);
 
-					if (pChan) {
-						sKey = pChan->GetKey();
-						pChan->SetIsOn(false);
-					}
-
-					PutIRC("JOIN " + sChan + " " + sKey);
+					// Don't try to rejoin!
+					pChan->Disable();
 				}
 
 				if ((pChan) && (pChan->IsDetached())) {
@@ -634,6 +695,15 @@ void CIRCSock::ReadLine(const CString& sData) {
 					m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + sTarget + " :" + sMsg);
 					return;
 				}
+			} else if (sCmd.CaseCmp("WALLOPS") == 0) {
+				// :blub!dummy@rox-8DBEFE92 WALLOPS :this is a test
+				CString sMsg = sRest.Token(0, true);
+
+				if(sMsg.Left(1) == ":") {
+					sMsg.LeftChomp();
+				}
+
+				m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " WALLOPS ", ":" + m_pUser->AddTimestamp(sMsg), false);
 			}
 		}
 	}
@@ -657,6 +727,11 @@ bool CIRCSock::OnCTCPReply(CNick& Nick, CString& sMessage) {
 }
 
 bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
+	if (sMessage.Left(7).CaseCmp("ACTION ") == 0) {
+		sMessage = sMessage.substr(7);
+		MODULECALL(OnPrivAction(Nick, sMessage), m_pUser, NULL, return true);
+		sMessage = "ACTION " + sMessage;
+	}
 	MODULECALL(OnPrivCTCP(Nick, sMessage), m_pUser, NULL, return true);
 
 	if (strncasecmp(sMessage.c_str(), "DCC ", 4) == 0 && m_pUser && m_pUser->BounceDCCs() && m_pUser->IsUserAttached()) {
@@ -681,7 +756,7 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 			}
 		} else if (sType.CaseCmp("RESUME") == 0) {
 			// Need to lookup the connection by port, filter the port, and forward to the user
-			CDCCBounce* pSock = (CDCCBounce*) CZNC::Get().GetManager().FindSockByLocalPort(atoi(sMessage.Token(3).c_str()));
+			CDCCBounce* pSock = (CDCCBounce*) CZNC::Get().GetManager().FindSockByLocalPort(sMessage.Token(3).ToUShort());
 
 			if ((pSock) && (strncasecmp(pSock->GetSockName().c_str(), "DCC::", 5) == 0)) {
 				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetUserPort()) + " " + sMessage.Token(4) + "\001");
@@ -706,6 +781,7 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 		const MCString& mssCTCPReplies = m_pUser->GetCTCPReplies();
 		MCString::const_iterator it = mssCTCPReplies.find(sMessage.AsUpper());
 		CString sQuery = sMessage.Token(0).AsUpper();
+		bool bHaveReply = false;
 		CString sReply;
 
 		if (it == mssCTCPReplies.end()) {
@@ -714,11 +790,12 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 
 		if (it != mssCTCPReplies.end()) {
 			sReply = it->second;
+			bHaveReply = true;
 		}
 
-		if (sReply.empty() && !m_pUser->IsUserAttached()) {
+		if (!bHaveReply && !m_pUser->IsUserAttached()) {
 			if (sQuery == "VERSION") {
-				sReply = "ZNC by prozac - http://znc.sourceforge.net";
+				sReply = CZNC::GetTag();
 			} else if (sQuery == "PING") {
 				sReply = sMessage.Token(1, true);
 			}
@@ -738,7 +815,7 @@ bool CIRCSock::OnPrivNotice(CNick& Nick, CString& sMessage) {
 
 	if (!m_pUser->IsUserAttached()) {
 		// If the user is detached, add to the buffer
-		m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " NOTICE ", " :" + sMessage);
+		m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " NOTICE ", " :" + m_pUser->AddTimestamp(sMessage));
 	}
 
 	return false;
@@ -749,7 +826,7 @@ bool CIRCSock::OnPrivMsg(CNick& Nick, CString& sMessage) {
 
 	if (!m_pUser->IsUserAttached()) {
 		// If the user is detached, add to the buffer
-		m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " PRIVMSG ", " :" + sMessage);
+		m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " PRIVMSG ", " :" + m_pUser->AddTimestamp(sMessage));
 	}
 
 	return false;
@@ -759,11 +836,15 @@ bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage)
 	CChan* pChan = m_pUser->FindChan(sChan);
 	if (pChan) {
 		// Record a /me
-		if (sMessage.Token(0).CaseCmp("ACTION") == 0 && (pChan->KeepBuffer() || !m_pUser->IsUserAttached())) {
-			pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :\001" + sMessage + "\001");
-		} else {
-			MODULECALL(OnChanCTCP(Nick, *pChan, sMessage), m_pUser, NULL, return true);
+		if (sMessage.Left(7).CaseCmp("ACTION ") == 0) {
+			sMessage = sMessage.substr(7);
+			if(pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
+				pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :\001ACTION " + m_pUser->AddTimestamp(sMessage) + "\001");
+			}
+			MODULECALL(OnChanAction(Nick, *pChan, sMessage), m_pUser, NULL, return true);
+			sMessage = "ACTION " + sMessage;
 		}
+		MODULECALL(OnChanCTCP(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 	}
 
 	return (pChan && pChan->IsDetached());
@@ -775,7 +856,7 @@ bool CIRCSock::OnChanNotice(CNick& Nick, const CString& sChan, CString& sMessage
 		MODULECALL(OnChanNotice(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 
 		if ((pChan->KeepBuffer()) || (!m_pUser->IsUserAttached())) {
-			pChan->AddBuffer(":" + Nick.GetNickMask() + " NOTICE " + sChan + " :" + sMessage);
+			pChan->AddBuffer(":" + Nick.GetNickMask() + " NOTICE " + sChan + " :" + m_pUser->AddTimestamp(sMessage));
 		}
 	}
 
@@ -788,7 +869,7 @@ bool CIRCSock::OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage) {
 		MODULECALL(OnChanMsg(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 
 		if (pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
-			pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :" + sMessage);
+			pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :" + m_pUser->AddTimestamp(sMessage));
 		}
 	}
 
@@ -821,24 +902,26 @@ void CIRCSock::Disconnected() {
 	MODULECALL(OnIRCDisconnected(), m_pUser, NULL, );
 
 	DEBUG_ONLY(cout << GetSockName() << " == Disconnected()" << endl);
-	if (!m_pUser->IsBeingDeleted()) {
-		m_pUser->PutStatus("Disconnected from IRC.  Reconnecting...");
+	if (!m_pUser->IsBeingDeleted() && m_pUser->GetIRCConnectEnabled()) {
+		m_pUser->PutStatus("Disconnected from IRC. Reconnecting...");
 	}
 	m_pUser->ClearRawBuffer();
 	m_pUser->ClearMotdBuffer();
 
 	ResetChans();
+	m_scUserModes.clear();
 }
 
 void CIRCSock::SockError(int iErrno) {
 	DEBUG_ONLY(cout << GetSockName() << " == SockError(" << iErrno << ")" << endl);
 	if (!m_pUser->IsBeingDeleted()) {
-		m_pUser->PutStatus("Disconnected from IRC.  Reconnecting...");
+		m_pUser->PutStatus("Disconnected from IRC (" + CString(strerror(iErrno)) + ").  Reconnecting...");
 	}
 	m_pUser->ClearRawBuffer();
 	m_pUser->ClearMotdBuffer();
 
 	ResetChans();
+	m_scUserModes.clear();
 }
 
 void CIRCSock::Timeout() {
@@ -850,6 +933,7 @@ void CIRCSock::Timeout() {
 	m_pUser->ClearMotdBuffer();
 
 	ResetChans();
+	m_scUserModes.empty();
 }
 
 void CIRCSock::ConnectionRefused() {
