@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _IRCSOCK_H
 #define _IRCSOCK_H
@@ -45,6 +51,7 @@ public:
 	void PutIRC(const CString& sLine);
 	void ParseISupport(const CString& sLine);
 	void ResetChans();
+	void Quit();
 
 	// Setters
 	void SetPass(const CString& s) { m_sPass = s; }
@@ -66,8 +73,10 @@ public:
 	const CString& GetNick() const { return m_Nick.GetNick(); }
 	const CString& GetPass() const { return m_sPass; }
 	bool IsOrigNickPending() const { return m_bOrigNickPending; }
+	CUser* GetUser() const { return m_pUser; }
 	bool HasNamesx() const { return m_bNamesx; }
 	bool HasUHNames() const { return m_bUHNames; }
+	const set<unsigned char>& GetUserModes() const { return m_scUserModes; }
 	// !Getters
 private:
 	void SetNick(const CString& sNick);
@@ -80,6 +89,7 @@ protected:
 	bool						m_bUHNames;
 	CString						m_sPerms;
 	CString						m_sPermModes;
+	set<unsigned char>			m_scUserModes;
 	map<unsigned char, EChanModeArgs>	m_mueChanModes;
 	CUser*						m_pUser;
 	CNick						m_Nick;
