@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifdef _MODULES
 
@@ -25,6 +31,11 @@ using std::set;
 #endif
 #endif
 
+#ifndef RTLD_LOCAL
+# define RTLD_LOCAL 0
+# warning "your crap box doesnt define RTLD_LOCAL !?"
+#endif
+
 #define MODCONSTRUCTOR(CLASS) \
 	CLASS(void *pDLL, CUser* pUser, const CString& sModName) : CModule(pDLL, pUser, sModName)
 #define MODULEDEFS(CLASS, DESCRIPTION) \
@@ -117,6 +128,9 @@ public:
 	CSocket(CModule* pModule, const CString& sLabel, const CString& sHostname, unsigned short uPort, int iTimeout = 60);
 	virtual ~CSocket();
 
+	using Csock::Connect;
+	using Csock::Listen;
+
 	bool Connect(const CString& sHostname, unsigned short uPort, bool bSSL = false, unsigned int uTimeout = 60);
 	bool Listen(unsigned short uPort, bool bSSL = false, unsigned int uTimeout = 0);
 	virtual bool PutIRC(const CString& sLine);
@@ -200,7 +214,7 @@ public:
 	void SetClient(CClient* pClient);
 	void Unload();
 
-	virtual bool OnLoad(const CString& sArgs);
+	virtual bool OnLoad(const CString& sArgsi, CString& sMessage);
 	virtual bool OnBoot();
 	virtual void OnIRCDisconnected();
 	virtual void OnIRCConnected();
@@ -233,6 +247,7 @@ public:
 	virtual EModRet OnUserRaw(CString& sLine);
 	virtual EModRet OnUserCTCPReply(CString& sTarget, CString& sMessage);
 	virtual EModRet OnUserCTCP(CString& sTarget, CString& sMessage);
+	virtual EModRet OnUserAction(CString& sTarget, CString& sMessage);
 	virtual EModRet OnUserMsg(CString& sTarget, CString& sMessage);
 	virtual EModRet OnUserNotice(CString& sTarget, CString& sMessage);
 	virtual EModRet OnUserJoin(CString& sChannel, CString& sKey);
@@ -241,6 +256,8 @@ public:
 	virtual EModRet OnCTCPReply(CNick& Nick, CString& sMessage);
 	virtual EModRet OnPrivCTCP(CNick& Nick, CString& sMessage);
 	virtual EModRet OnChanCTCP(CNick& Nick, CChan& Channel, CString& sMessage);
+	virtual EModRet OnPrivAction(CNick& Nick, CString& sMessage);
+	virtual EModRet OnChanAction(CNick& Nick, CChan& Channel, CString& sMessage);
 	virtual EModRet OnPrivMsg(CNick& Nick, CString& sMessage);
 	virtual EModRet OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage);
 	virtual EModRet OnPrivNotice(CNick& Nick, CString& sMessage);
@@ -361,6 +378,7 @@ public:
 	virtual bool OnUserRaw(CString& sLine);
 	virtual bool OnUserCTCPReply(CString& sTarget, CString& sMessage);
 	virtual bool OnUserCTCP(CString& sTarget, CString& sMessage);
+	virtual bool OnUserAction(CString& sTarget, CString& sMessage);
 	virtual bool OnUserMsg(CString& sTarget, CString& sMessage);
 	virtual bool OnUserNotice(CString& sTarget, CString& sMessage);
 	virtual bool OnUserJoin(CString& sChannel, CString& sKey);
@@ -369,6 +387,8 @@ public:
 	virtual bool OnCTCPReply(CNick& Nick, CString& sMessage);
 	virtual bool OnPrivCTCP(CNick& Nick, CString& sMessage);
 	virtual bool OnChanCTCP(CNick& Nick, CChan& Channel, CString& sMessage);
+	virtual bool OnPrivAction(CNick& Nick, CString& sMessage);
+	virtual bool OnChanAction(CNick& Nick, CChan& Channel, CString& sMessage);
 	virtual bool OnPrivMsg(CNick& Nick, CString& sMessage);
 	virtual bool OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage);
 	virtual bool OnPrivNotice(CNick& Nick, CString& sMessage);
@@ -394,6 +414,7 @@ public:
 	virtual ~CGlobalModule() {}
 
 	virtual EModRet OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan);
+	virtual void OnFinishedConfig();
 	virtual EModRet OnDeleteUser(CUser& User);
 	virtual EModRet OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
 	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
@@ -406,6 +427,7 @@ public:
 	virtual ~CGlobalModules() {}
 
 	virtual bool OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan);
+	virtual void OnFinishedConfig();
 	virtual bool OnDeleteUser(CUser& User);
 	virtual bool OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
 	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
