@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "Chan.h"
 #include "Nick.h"
@@ -116,7 +122,7 @@ void CNick::UpdatePermChar() {
 }
 
 const set<unsigned char>& CNick::GetChanPerms() const { return m_suChanPerms; }
-const unsigned char CNick::GetPermChar() const { return m_cPerm; }
+unsigned char CNick::GetPermChar() const { return m_cPerm; }
 CString CNick::GetPermStr() const {
 	CIRCSock* pIRCSock = (!m_pUser) ? NULL : m_pUser->GetIRCSock();
 	const CString& sChanPerms = (!pIRCSock) ? "@+" : pIRCSock->GetPerms();
