@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _NICK_H
 #define _NICK_H
@@ -41,7 +47,7 @@ public:
 	// Getters
 	const set<unsigned char>& GetChanPerms() const;
 	CString GetPermStr() const;
-	const unsigned char GetPermChar() const;
+	unsigned char GetPermChar() const;
 	bool HasPerm(unsigned char uPerm) const;
 	const CString& GetNick() const;
 	const CString& GetIdent() const;
