@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "Server.h"
@@ -36,5 +42,5 @@ bool CServer::IsSSL() const { return m_bSSL; }
 bool CServer::IsIPV6() const { return m_bIPV6; }
 
 CString CServer::GetString() const {
-	return m_sName + " " + CString(m_bSSL ? "+" : "") + CString(m_uPort) + CString(m_sPass.empty() ? CString("") : " " + m_sPass);
+	return m_sName + " " + CString(m_bSSL ? "+" : "") + CString(m_uPort) + CString(m_sPass.empty() ? "" : " " + m_sPass);
 }
