@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef X_STRING_H
 #define X_STRING_H
@@ -7,6 +13,8 @@
 #include <pcre.h>
 #endif
 
+#include <sys/types.h>
+
 #include <set>
 #include <map>
 #include <string>
