@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _TIMERS_H
 #define _TIMERS_H
@@ -10,6 +16,7 @@ public:
 	CKeepNickTimer(CUser* pUser) : CCron() {
 		m_pUser = pUser;
 		m_uTrys = 0;
+		SetName("CKeepNickTimer::" + m_pUser->GetUserName());
 		Start(5);
 	}
 	virtual ~CKeepNickTimer() {}
@@ -37,6 +44,7 @@ class CMiscTimer : public CCron {
 public:
 	CMiscTimer(CUser* pUser) : CCron() {
 		m_pUser = pUser;
+		SetName("CMiscTimer::" + m_pUser->GetUserName());
 		Start(30);
 	}
 	virtual ~CMiscTimer() {}
@@ -47,14 +55,14 @@ protected:
 		vector<CClient*>& vClients = m_pUser->GetClients();
 		CIRCSock* pIRCSock = m_pUser->GetIRCSock();
 
-		if (pIRCSock && pIRCSock->GetTimeSinceLastWrite() >= 270) {
+		if (pIRCSock && pIRCSock->GetTimeSinceLastDataTransaction() >= 270) {
 			pIRCSock->PutIRC("PING :ZNC");
 		}
 
 		for (size_t a = 0; a < vClients.size(); a++) {
 			CClient* pClient = vClients[a];
 
-			if (pClient->GetTimeSinceLastWrite() >= 470) {
+			if (pClient->GetTimeSinceLastDataTransaction() >= 470) {
 				pClient->PutClient("PING :ZNC");
 			}
 		}
@@ -67,6 +75,7 @@ class CJoinTimer : public CCron {
 public:
 	CJoinTimer(CUser* pUser) : CCron() {
 		m_pUser = pUser;
+		SetName("CJoinTimer::" + m_pUser->GetUserName());
 		Start(20);
 	}
 	virtual ~CJoinTimer() {}
