@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "znc.h"
@@ -12,12 +18,15 @@
 #include "Timers.h"
 
 CUser::CUser(const CString& sUserName) {
+	m_fTimezoneOffset = 0;
 	m_uConnectTime = 0;
 	SetUserName(sUserName);
 	m_sNick = m_sCleanUserName;
 	m_sIdent = m_sCleanUserName;
 	m_sRealName = sUserName;
 	m_uServerIdx = 0;
+	m_uBytesRead = 0;
+	m_uBytesWritten = 0;
 #ifdef _MODULES
 	m_pModules = new CModules;
 #endif
@@ -34,9 +43,14 @@ CUser::CUser(const CString& sUserName) {
 	m_sStatusPrefix = "*";
 	m_sChanPrefixes = "#&";
 	m_uBufferCount = 50;
+	m_uMaxJoinTries = 0;
 	m_bKeepBuffer = false;
 	m_bAutoCycle = true;
 	m_bBeingDeleted = false;
+	m_sTimestampFormat = "[%H:%M:%S]";
+	m_bAppendTimestamp = false;
+	m_bPrependTimestamp = true;
+	m_bIRCConnectEnabled = true;
 	m_pKeepNickTimer = new CKeepNickTimer(this);
 	m_pJoinTimer = new CJoinTimer(this);
 	m_pMiscTimer = new CMiscTimer(this);
@@ -105,6 +119,9 @@ void CUser::IRCDisconnected() {
 	for (unsigned int a = 0; a < m_vClients.size(); a++) {
 		m_vClients[a]->IRCDisconnected();
 	}
+
+	// Get the reconnect going
+	CZNC::Get().EnableConnectUser();
 }
 
 CString CUser::ExpandString(const CString& sStr) const {
@@ -125,6 +142,35 @@ CString& CUser::ExpandString(const CString& sStr, CString& sRet) const {
 	return sRet;
 }
 
+CString CUser::AddTimestamp(const CString& sStr) const {
+	CString sRet;
+	return AddTimestamp(sStr, sRet);
+}
+
+CString& CUser::AddTimestamp(const CString& sStr, CString& sRet) const {
+	char szTimestamp[1024];
+	time_t tm;
+
+	if(GetTimestampFormat().empty() || (!m_bAppendTimestamp && !m_bPrependTimestamp)) {
+		sRet = sStr;
+	} else {
+		time(&tm);
+		tm += (time_t)(m_fTimezoneOffset * 60 * 60); // offset is in hours
+		strftime(szTimestamp, sizeof(szTimestamp) / sizeof(char), GetTimestampFormat().c_str(), localtime(&tm));
+
+		sRet = sStr;
+		if (m_bPrependTimestamp) {
+			sRet = szTimestamp;
+			sRet += " " + sStr;
+		}
+		if (m_bAppendTimestamp) {
+			sRet += " ";
+			sRet += szTimestamp;
+		} 
+	}
+	return sRet;
+}
+
 void CUser::BounceAllClients() {
 	for (unsigned int a = 0; a < m_vClients.size(); a++) {
 		m_vClients[a]->BouncedOff();
@@ -164,6 +210,18 @@ void CUser::UserConnected(CClient* pClient) {
 		}
 	}
 
+	if(GetIRCSock() != NULL) {
+		CString sUserMode("");
+		const set<unsigned char>& scUserModes = GetIRCSock()->GetUserModes();
+		for (set<unsigned char>::iterator it = scUserModes.begin();
+				it != scUserModes.end(); it++) {
+			sUserMode += *it;
+		}
+		if(!sUserMode.empty()) {
+			pClient->PutClient(":" + GetIRCNick().GetNick() + " MODE " + GetIRCNick().GetNick() + " :+" + sUserMode);
+		}
+	}
+
 	const vector<CChan*>& vChans = GetChans();
 	for (unsigned int a = 0; a < vChans.size(); a++) {
 		if ((vChans[a]->IsOn()) && (!vChans[a]->IsDetached())) {
@@ -175,6 +233,11 @@ void CUser::UserConnected(CClient* pClient) {
 	while (m_QueryBuffer.GetNextLine(GetIRCNick().GetNick(), sBufLine)) {
 		pClient->PutClient(sBufLine);
 	}
+
+	// Tell them why they won't connect
+	if (!GetIRCConnectEnabled())
+		PutStatus("You are currently disconnected from IRC. "
+				"Use 'connect' to reconnect.");
 }
 
 void CUser::UserDisconnected(CClient* pClient) {
@@ -300,7 +363,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 
 			if (pSock) {
 				PutStatus("Jumping servers because this server is no longer in the list");
-				pSock->Close();
+				pSock->Quit();
 			}
 		}
 	}
@@ -347,6 +410,9 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 	SetUseClientIP(User.UseClientIP());
 	SetDenyLoadMod(User.DenyLoadMod());
 	SetAdmin(User.IsAdmin());
+	SetTimestampAppend(User.GetTimestampAppend());
+	SetTimestampPrepend(User.GetTimestampPrepend());
+	SetTimestampFormat(User.GetTimestampFormat());
 	// !Flags
 
 	return true;
@@ -376,6 +442,10 @@ bool CUser::IsHostAllowed(const CString& sHostMask) {
 	return false;
 }
 
+const CString& CUser::GetTimestampFormat() const { return m_sTimestampFormat; }
+bool CUser::GetTimestampAppend() const { return m_bAppendTimestamp; }
+bool CUser::GetTimestampPrepend() const { return m_bPrependTimestamp; }
+
 bool CUser::IsValidUserName(const CString& sUserName) {
 	const char* p = sUserName.c_str();
 
@@ -448,6 +518,7 @@ bool CUser::AddChan(const CString& sName, bool bInConfig) {
 bool CUser::DelChan(const CString& sName) {
 	for (vector<CChan*>::iterator a = m_vChans.begin(); a != m_vChans.end(); a++) {
 		if (sName.CaseCmp((*a)->GetName()) == 0) {
+			delete *a;
 			m_vChans.erase(a);
 			return true;
 		}
@@ -485,6 +556,11 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "DenyLoadMod", CString((DenyLoadMod()) ? "true" : "false"));
 	PrintLine(File, "Admin", CString((IsAdmin()) ? "true" : "false"));
 	PrintLine(File, "DCCLookupMethod", CString((UseClientIP()) ? "client" : "default"));
+	PrintLine(File, "TimestampFormat", GetTimestampFormat());
+	PrintLine(File, "AppendTimestamp", CString((GetTimestampAppend()) ? "true" : "false"));
+	PrintLine(File, "PrependTimestamp", CString((GetTimestampPrepend()) ? "true" : "false"));
+	PrintLine(File, "TimezoneOffset", CString(m_fTimezoneOffset));
+	PrintLine(File, "JoinTries", CString(m_uMaxJoinTries));
 	File.Write("\r\n");
 
 	// Allow Hosts
@@ -559,8 +635,14 @@ CChan* CUser::FindChan(const CString& sName) const {
 void CUser::JoinChans() {
 	for (unsigned int a = 0; a < m_vChans.size(); a++) {
 		CChan* pChan = m_vChans[a];
-		if (!pChan->IsOn()) {
-			PutIRC("JOIN " + pChan->GetName() + " " + pChan->GetKey());
+		if (!pChan->IsOn() && !pChan->IsDisabled()) {
+			if (JoinTries() != 0 && pChan->GetJoinTries() >= JoinTries()) {
+				PutStatus("The channel " + pChan->GetName() + " could not be joined, disabling it.");
+				pChan->Disable();
+			} else {
+				pChan->IncJoinTries();
+				PutIRC("JOIN " + pChan->GetName() + " " + pChan->GetKey());
+			}
 		}
 	}
 }
@@ -598,8 +680,8 @@ bool CUser::DelServer(const CString& sName) {
 				}
 
 				if (pIRCSock) {
-					pIRCSock->Close();
-					PutUser("Your current server was removed, jumping...");
+					pIRCSock->Quit();
+					PutStatus("Your current server was removed, jumping...");
 				}
 			} else if (m_uServerIdx >= m_vServers.size()) {
 				m_uServerIdx = 0;
@@ -661,6 +743,8 @@ bool CUser::AddServer(const CString& sName, unsigned short uPort, const CString&
 	CServer* pServer = new CServer(sName, uPort, sPass, bSSL, bIPV6);
 	m_vServers.push_back(pServer);
 
+	CheckIRCConnect();
+
 	return true;
 }
 
@@ -838,7 +922,7 @@ bool CUser::SendFile(const CString& sRemoteNick, const CString& sFileName, const
 		return false;
 	}
 
-	unsigned short uPort = CZNC::Get().GetManager().ListenAllRand("DCC::LISTEN::" + sRemoteNick, false, SOMAXCONN, pSock, 120);
+	unsigned short uPort = CZNC::Get().GetManager().ListenRand("DCC::LISTEN::" + sRemoteNick, GetLocalIP(), false, SOMAXCONN, pSock, 120);
 
 	if (GetNick().CaseCmp(sRemoteNick) == 0) {
 		PutUser(":" + GetStatusPrefix() + "status!znc@znc.com PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
@@ -917,6 +1001,13 @@ void CUser::SetBufferCount(unsigned int u) { m_uBufferCount = u; }
 void CUser::SetKeepBuffer(bool b) { m_bKeepBuffer = b; }
 void CUser::SetAutoCycle(bool b) { m_bAutoCycle = b; }
 
+void CUser::CheckIRCConnect()
+{
+	// Do we want to connect?
+	if (m_bIRCConnectEnabled && GetIRCSock() == NULL)
+		CZNC::Get().EnableConnectUser();
+}
+
 void CUser::SetIRCNick(const CNick& n) {
 	m_IRCNick = n;
 
@@ -926,7 +1017,7 @@ void CUser::SetIRCNick(const CNick& n) {
 }
 
 bool CUser::AddCTCPReply(const CString& sCTCP, const CString& sReply) {
-	if (sCTCP.empty() || sReply.empty()) {
+	if (sCTCP.empty()) {
 		return false;
 	}
 
@@ -958,12 +1049,12 @@ bool CUser::IsPassHashed() const { return m_bPassHashed; }
 bool CUser::ConnectPaused() {
 	if (!m_uConnectTime) {
 		m_uConnectTime = time(NULL);
-		return false;
+		return !m_bIRCConnectEnabled;
 	}
 
 	if (time(NULL) - m_uConnectTime >= 5) {
 		m_uConnectTime = time(NULL);
-		return false;
+		return !m_bIRCConnectEnabled;
 	}
 
 	return true;
@@ -981,7 +1072,7 @@ const vector<CChan*>& CUser::GetChans() const { return m_vChans; }
 const vector<CServer*>& CUser::GetServers() const { return m_vServers; }
 const CNick& CUser::GetIRCNick() const { return m_IRCNick; }
 const CString& CUser::GetIRCServer() const { return m_sIRCServer; }
-CString CUser::GetQuitMsg() const { return (!m_sQuitMsg.empty()) ? m_sQuitMsg : "ZNC by prozac - http://znc.sourceforge.net"; }
+CString CUser::GetQuitMsg() const { return (!m_sQuitMsg.empty()) ? m_sQuitMsg : CZNC::GetTag(false); }
 const MCString& CUser::GetCTCPReplies() const { return m_mssCTCPReplies; }
 unsigned int CUser::GetBufferCount() const { return m_uBufferCount; }
 bool CUser::KeepBuffer() const { return m_bKeepBuffer; }
