@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _USER_H
 #define _USER_H
@@ -66,9 +72,12 @@ public:
 #endif
 
 	// Buffers
-	void AddRawBuffer(const CString& sPre, const CString& sPost) { m_RawBuffer.AddLine(sPre, sPost); }
-	void AddMotdBuffer(const CString& sPre, const CString& sPost) { m_MotdBuffer.AddLine(sPre, sPost); }
-	void AddQueryBuffer(const CString& sPre, const CString& sPost) { m_QueryBuffer.AddLine(sPre, sPost); }
+	void AddRawBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_RawBuffer.AddLine(sPre, sPost, bIncNick); }
+	void AddMotdBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_MotdBuffer.AddLine(sPre, sPost, bIncNick); }
+	void AddQueryBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_QueryBuffer.AddLine(sPre, sPost, bIncNick); }
+	void UpdateRawBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_RawBuffer.UpdateLine(sPre, sPost, bIncNick); }
+	void UpdateMotdBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_MotdBuffer.UpdateLine(sPre, sPost, bIncNick); }
+	void UpdateQueryBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_QueryBuffer.UpdateLine(sPre, sPost, bIncNick); }
 	void ClearRawBuffer() { m_RawBuffer.Clear(); }
 	void ClearMotdBuffer() { m_MotdBuffer.Clear(); }
 	void ClearQueryBuffer() { m_QueryBuffer.Clear(); }
@@ -89,10 +98,14 @@ public:
 	bool IsIRCConnected() { return m_bIRCConnected; }
 	void IRCConnected(CIRCSock* pIRCSock);
 	void IRCDisconnected();
+	void CheckIRCConnect();
 
 	CString ExpandString(const CString& sStr) const;
 	CString& ExpandString(const CString& sStr, CString& sRet) const;
 
+	CString AddTimestamp(const CString& sStr) const;
+	CString& AddTimestamp(const CString& sStr, CString& sRet) const;
+
 	bool SendFile(const CString& sRemoteNick, const CString& sFileName, const CString& sModuleName = "");
 	bool GetFile(const CString& sRemoteNick, const CString& sRemoteIP, unsigned short uRemotePort, const CString& sFileName, unsigned long uFileSize, const CString& sModuleName = "");
 	bool ResumeFile(const CString& sRemoteNick, unsigned short uPort, unsigned long uFileSize);
@@ -100,6 +113,9 @@ public:
 	bool Clone(const CUser& User, CString& sErrorRet);
 	void BounceAllClients();
 
+	void AddBytesRead(unsigned long long u) { m_uBytesRead += u; }
+	void AddBytesWritten(unsigned long long u) { m_uBytesWritten += u; }
+
 	// Setters
 	void SetUserName(const CString& s);
 	void SetNick(const CString& s);
@@ -125,6 +141,12 @@ public:
 	void SetAutoCycle(bool b);
 	void SetChanPrefixes(const CString& s) { m_sChanPrefixes = (s.empty()) ? "#&" : s; }
 	void SetBeingDeleted(bool b) { m_bBeingDeleted = b; }
+	void SetTimestampFormat(const CString& s) { m_sTimestampFormat = s; }
+	void SetTimestampAppend(bool b) { m_bAppendTimestamp = b; }
+	void SetTimestampPrepend(bool b) { m_bPrependTimestamp = b; }
+	void SetTimezoneOffset(float b) { m_fTimezoneOffset = b; }
+	void SetJoinTries(unsigned int i) { m_uMaxJoinTries = i; }
+	void SetIRCConnectEnabled(bool b) { m_bIRCConnectEnabled = b; }
 	// !Setters
 
 	// Getters
@@ -141,6 +163,10 @@ public:
 	const CString& GetPass() const;
 	bool IsPassHashed() const;
 	const set<CString>& GetAllowedHosts() const;
+	const CString& GetTimestampFormat() const;
+	bool GetTimestampAppend() const;
+	bool GetTimestampPrepend() const;
+	bool GetIRCConnectEnabled() const { return m_bIRCConnectEnabled; }
 
 	const CString& GetChanPrefixes() const { return m_sChanPrefixes; }
 	bool IsChan(const CString& sChan) const { return (sChan.size() && GetChanPrefixes().find(sChan[0]) != CString::npos); }
@@ -167,6 +193,10 @@ public:
 	bool AutoCycle() const;
 	bool IsBeingDeleted() const { return m_bBeingDeleted; }
 	bool HasServers() const { return m_vServers.size() > 0; }
+	float GetTimezoneOffset() const { return m_fTimezoneOffset; }
+	unsigned long long BytesRead() const { return m_uBytesRead; }
+	unsigned long long BytesWritten() const { return m_uBytesWritten; }
+	unsigned int JoinTries() const { return m_uMaxJoinTries; }
 	// !Getters
 private:
 protected:
@@ -186,6 +216,8 @@ protected:
 	CString			m_sIRCServer;
 	CString			m_sQuitMsg;
 	MCString		m_mssCTCPReplies;
+	CString			m_sTimestampFormat;
+	float			m_fTimezoneOffset;
 
 	// Paths
 	CString			m_sUserPath;
@@ -206,6 +238,9 @@ protected:
 	bool				m_bKeepBuffer;
 	bool				m_bAutoCycle;
 	bool				m_bBeingDeleted;
+	bool				m_bAppendTimestamp;
+	bool				m_bPrependTimestamp;
+	bool				m_bIRCConnectEnabled;
 
 	CKeepNickTimer*		m_pKeepNickTimer;
 	CJoinTimer*			m_pJoinTimer;
@@ -217,6 +252,9 @@ protected:
 	set<CString>		m_ssAllowedHosts;
 	unsigned int		m_uServerIdx;
 	unsigned int		m_uBufferCount;
+	unsigned long long      m_uBytesRead;
+	unsigned long long      m_uBytesWritten;
+	unsigned int		m_uMaxJoinTries;
 
 #ifdef _MODULES
 	CModules*		m_pModules;
