@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _UTILS_H
 #define _UTILS_H
@@ -432,7 +438,7 @@ class CSafePtr : private CNoCopy {
 public:
 	CSafePtr() { m_pType = new T; }
 	CSafePtr(T* p) { m_pType = p; }
-	~CSafePtr() { delete m_pType; }
+	virtual ~CSafePtr() { delete m_pType; }
 
 	T& operator *() const { assert(m_pType); return *m_pType; }
 	T* operator ->() const { assert(m_pType); return m_pType; }
