@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include <unistd.h>
 #include <getopt.h>
@@ -32,7 +38,7 @@ void GenerateHelp(const char *appname) {
 	CUtils::PrintMessage("\t-p, --makepem      Generates a pemfile for use with SSL");
 	CUtils::PrintMessage("\t-e, --encrypt-pem  when used along with --makepem, encrypts the private key in the pemfile");
 #endif /* HAVE_LIBSSL */
-	CUtils::PrintMessage("\t-d, --datadir      Set a different znc repository");
+	CUtils::PrintMessage("\t-d, --datadir      Set a different znc repository (default is ~/.znc)");
 }
 
 void die(int sig) {
@@ -55,6 +61,8 @@ int main(int argc, char** argv, char** envp) {
 	CString sConfig;
 	CString sDataDir = "";
 
+	srand(time(NULL));
+
 #ifdef HAVE_LIBSSL
 	InitSSL();
 #endif /* HAVE_LIBSSL */
