@@ -1,9 +1,16 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _MAIN_H
 #define _MAIN_H
 
-#define VERSION 0.047
+// Keep the number in sync with configure.in (and also with configure)
+#define VERSION 0.052
 
 #ifndef _MODDIR_
 #define _MODDIR_ "/usr/libexec/znc"
@@ -46,6 +53,10 @@ using std::cout;
 using std::cerr;
 using std::endl;
 
+#ifndef PATH_MAX
+#define PATH_MAX	1024
+#endif
+
 #include "String.h"
 #include "Csocket.h"
 #include "Utils.h"
