@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "User.h"
@@ -126,7 +132,7 @@ class CAutoOpMod : public CModule {
 public:
 	MODCONSTRUCTOR(CAutoOpMod) {}
 
-	virtual bool OnLoad(const CString& sArgs) {
+	virtual bool OnLoad(const CString& sArgs, CString& sMessage) {
 		AddTimer(new CAutoOpTimer(this));
 
 		// Load the users
