@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 #define REQUIRESSL
 
 #include "main.h"
@@ -17,7 +25,7 @@
  * Author: imaginos <imaginos@imaginos.net>
  *
  * 
- * $Log: away.cpp,v $
+ * $Log$
  * Revision 1.25  2006/09/16 18:11:35  prozacx
  * Patched using configureable_away_timer.patch by psycho
  *
@@ -127,7 +135,7 @@ public:
 			SaveBufferToDisk();
 	}
 
-	virtual bool OnLoad(const CString& sArgs)
+	virtual bool OnLoad(const CString& sArgs, CString& sMessage)
 	{
 		CString sMyArgs = sArgs;
 		if (sMyArgs.Token(0) == "-notimer")
@@ -427,7 +435,7 @@ public:
 		return( CONTINUE );	
 	}
 	
-	virtual EModRet OnUserNotice(const CString& sTarget, CString& sMessage)
+	virtual EModRet OnUserNotice(CString& sTarget, CString& sMessage)
 	{
 		Ping();
 		if( m_bIsAway )
@@ -484,6 +492,8 @@ private:
 
 	void AddMessage( time_t iTime, const CNick & Nick, CString & sMessage )
 	{
+		if (m_pUser && Nick.GetNick() == m_pUser->GetIRCNick().GetNick())
+			return; // ignore messages from self
 		AddMessage( CString( iTime ) + ":" + Nick.GetNickMask() + ":" + sMessage );
 	}
 
