@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "znc.h"
@@ -13,10 +19,6 @@ public:
 		m_bWriteConf = false;
 	}
 
-	virtual bool OnLoad(const CString& sArgs) {
-		return true;
-	}
-
 	virtual ~CChanSaverMod() {
 	}
 
