@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 //! @author prozac@rottenboy.com
 // 
 // The encryption here was designed to be compatible with mircryption's CBC mode.
@@ -26,15 +34,7 @@
 class CCryptMod : public CModule {
 public:
 	MODCONSTRUCTOR(CCryptMod) {}
-	virtual ~CCryptMod() {
-		time_t t;
-		time(&t);
-		srand((long) t);
-	}
-
-	virtual EModRet OnRaw(CString& sLine) {
-		return CONTINUE;
-	}
+	virtual ~CCryptMod() {}
 
 	virtual EModRet OnUserMsg(CString& sTarget, CString& sMessage) {
 		sTarget.TrimLeft("\244");
