@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 #include "znc.h"
 #include "main.h"
 #include "User.h"
@@ -15,7 +23,7 @@
  * Email Monitor / Retrieval
  * Author: imaginos <imaginos@imaginos.net>
  *
- * $Log: email.cpp,v $
+ * $Log$
  * Revision 1.12  2006/02/25 09:43:35  prozacx
  * Migrated away from CString::ToString() in favor of explicit constructors
  *
@@ -93,7 +101,7 @@ public:
 			m_pManager->DelSockByAddr( vSocks[a] );
 	}
 
-	virtual bool OnLoad(const CString & sArgs) {
+	virtual bool OnLoad(const CString & sArgs, CString& sMessage) {
 		m_sMailPath = sArgs;
 
 		StartParser();
