@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "User.h"
@@ -55,7 +61,7 @@ public:
 		return true;
 	}
 
-	virtual bool OnLoad(const CString& sArgs) {
+	virtual bool OnLoad(const CString& sArgs, CString& sMessage) {
 		if (sArgs.Trim_n().empty()) {
 			return true;	// use defaults
 		}
@@ -152,4 +158,4 @@ void CIMAPSock::ReadLine(const CString& sLine) {
 	}
 }
 
-GLOBALMODULEDEFS(CIMAPAuthMod, "Allow users to authenticate via imap");
+GLOBALMODULEDEFS(CIMAPAuthMod, "Allow users to authenticate via imap")
