@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 #ifdef HAVE_PERL
 #include "main.h"
 #include "User.h"
@@ -287,7 +295,7 @@ public:
 	}
 	void UnSetUser() { m_pUser = NULL; }
 
-	virtual bool OnLoad( const CString & sArgs );
+	virtual bool OnLoad( const CString & sArgs, CString & sMessage );
 	virtual void OnUserAttached() {  CBNone( "OnUserAttached" ); }
 	virtual void OnUserDetached() {  CBNone( "OnUserDetached" ); }
 	virtual void OnIRCDisconnected() {  CBNone( "OnIRCDisconnected" ); }
@@ -365,7 +373,7 @@ public:
 	{
 		return CBDouble( "OnCTCPReply", NICK( Nick ), sMessage ); 
 	}
-	virtual EModRet OnUserCTCP(const CString& sTarget, CString& sMessage)
+	virtual EModRet OnUserCTCP(CString& sTarget, CString& sMessage)
 	{
 		return CBDouble( "OnUserCTCP", sTarget, sMessage ); 
 	}
@@ -390,7 +398,7 @@ public:
 	{
 		return( CBTriple( "OnChanMsg", NICK( Nick ), CHAN( Channel ), sMessage ) );
 	}
-	virtual EModRet OnUserNotice(const CString& sTarget, CString& sMessage)
+	virtual EModRet OnUserNotice(CString& sTarget, CString& sMessage)
 	{
 		return CBDouble( "OnUserNotice", sTarget, sMessage );
 	}
@@ -968,7 +976,7 @@ CModPerl::EModRet CModPerl::CallBack( const PString & sHookName, const VPString
 // special case this, required for perl modules that are dynamic
 EXTERN_C void boot_DynaLoader (pTHX_ CV* cv);
 
-bool CModPerl::OnLoad( const CString & sArgs ) 
+bool CModPerl::OnLoad( const CString & sArgs, CString & sMessage ) 
 {
 	m_pPerl = perl_alloc();
 	perl_construct( m_pPerl );
