@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 #include "main.h"
 #include "User.h"
 #include "Nick.h"
@@ -19,7 +27,7 @@ public:
 	{
 	}
 
-	virtual bool OnLoad(const CString& sArgs)
+	virtual bool OnLoad(const CString& sArgs, CString& sMessage)
 	{
 		GetNV("Perform").Split("\n", m_vPerform, false);
 		
@@ -75,15 +83,9 @@ public:
 					PutModule(CString( i ) + ": " + *it);
 			}
 			PutModule(" -- End of List");
-		} else if(sCmdName == "save")
-		{
-			if(Save())
-				PutModule("Saved!");
-			else
-				PutModule("Error");
 		}else
 		{
-			PutModule( "Commands: add <command>, del <nr>, list, save");
+			PutModule( "Commands: add <command>, del <nr>, list");
 		}
 	}
 
