@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "User.h"
@@ -23,7 +29,7 @@ class CSampleMod : public CModule {
 public:
 	MODCONSTRUCTOR(CSampleMod) {}
 
-	virtual bool OnLoad(const CString& sArgs) {
+	virtual bool OnLoad(const CString& sArgs, CString& sMessage) {
 		PutModule("I'm being loaded with the arguments: [" + sArgs + "]");
 		//AddTimer(new CSampleTimer(this, 300, 0, "Sample", "Sample timer for sample things."));
 		//AddTimer(new CSampleTimer(this, 5, 20, "Another", "Another sample timer."));
@@ -141,7 +147,7 @@ public:
 		return CONTINUE;
 	}
 
-	virtual EModRet OnUserNotice(const CString& sTarget, CString& sMessage) {
+	virtual EModRet OnUserNotice(CString& sTarget, CString& sMessage) {
 		PutModule("[" + sTarget + "] usernotice [" + sMessage + "]");
 		sMessage = "\037" + sMessage + "\037";
 
