@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 #define REQUIRESSL
 
 #include "main.h"
@@ -23,7 +31,7 @@
  * Its only as secure as your shell, the encryption only offers a slightly 
  * better solution then plain text.
  * 
- * $Log: savebuff.cpp,v $
+ * $Log$
  * Revision 1.29  2006/02/25 09:43:35  prozacx
  * Migrated away from CString::ToString() in favor of explicit constructors
  *
@@ -154,7 +162,7 @@ public:
 		}
 	}
 
-	virtual bool OnLoad(const CString& sArgs)
+	virtual bool OnLoad(const CString& sArgs, CString& sMessage)
 	{
 		if (!sArgs.empty())
 		{
