@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 #define REQUIRESSL
 
 #include "main.h"
@@ -20,7 +28,7 @@ using std::pair;
  * Secure chat system
  * Author: imaginos <imaginos@imaginos.net>
  * 
- * $Log: schat.cpp,v $
+ * $Log$
  * Revision 1.23  2006/02/25 09:43:35  prozacx
  * Migrated away from CString::ToString() in favor of explicit constructors
  *
@@ -201,7 +209,7 @@ public:
 	MODCONSTRUCTOR(CSChat) {}
 	virtual ~CSChat() { CleanSocks(); }
 
-	virtual bool OnLoad( const CString & sArgs )
+	virtual bool OnLoad( const CString & sArgs, CString & sMessage )
 	{
 		m_sPemFile = sArgs;
 
@@ -211,7 +219,7 @@ public:
 		}
 
 		if (!CFile::Exists(m_sPemFile)) {
-			PutModule("Unable to load pem file [" + m_sPemFile + "]");
+			sMessage = "Unable to load pem file [" + m_sPemFile + "]";
 			return false;
 		}
 
