@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "Csocket.h"
@@ -140,7 +146,7 @@ private:
 void CShellSock::ReadLine(const CString& sData) {
 	CString sLine = sData;
 
-	while ((sLine.length()) && (sLine[sLine.length() -1] == '\r') || (sLine[sLine.length() -1] == '\n')) {
+	while (sLine.length() && (sLine[sLine.length() -1] == '\r' || sLine[sLine.length() -1] == '\n')) {
 		sLine = sLine.substr(0, sLine.length() -1);
 	}
 
