@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
+
 #include "main.h"
 #include "User.h"
 #include "Nick.h"
@@ -9,7 +17,6 @@
 #include <map>
 #include <vector>
 
-
 class CStickyChan : public CModule 
 {
 public:
@@ -18,7 +25,7 @@ public:
 	{
 	}
 
-	virtual bool OnLoad( const CString& sArgs );
+	virtual bool OnLoad( const CString& sArgs, CString& sMessage );
 
 	virtual EModRet OnUserPart( CString& sChannel, CString& sMessage )
 	{
@@ -56,10 +63,22 @@ public:
 				DelNV( it );
 
 			PutModule( "UnStuck " + sChannel );
-		}	
+		}
+		else if ( ( sCmdName == "list" ) && ( sChannel.empty() ) )
+		{
+			int i = 1;
+			for( MCString::iterator it = BeginNV(); it != EndNV(); it++, i++ )
+			{
+				if (it->second.empty())
+					PutModule(CString( i ) + ": " + it->first);
+				else
+					PutModule(CString( i ) + ": " + it->first + " (" + it->second + ")");
+			}
+			PutModule(" -- End of List");
+		}
 		else
 		{
-			PutModule( "USAGE: [un]stick #channel [key]" );
+			PutModule( "USAGE: [un]stick #channel [key], list" );
 		}
 	}
 
@@ -88,7 +107,7 @@ static void RunTimer( CModule * pModule, CFPTimer *pTimer )
 	((CStickyChan *)pModule)->RunJob();
 }
 
-bool CStickyChan::OnLoad(const CString& sArgs)
+bool CStickyChan::OnLoad(const CString& sArgs, CString& sMessage)
 {
 	AddTimer( RunTimer, "StickyChanTimer", 15 );
 	return( true );
