@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "main.h"
 #include "User.h"
@@ -115,7 +121,7 @@ public:
 		return true;
 	}
 
-	virtual bool OnLoad(const CString& sArgStr) {
+	virtual bool OnLoad(const CString& sArgStr, CString& sMessage) {
 		bool bSSL = false;
 		bool bIPv6 = false;
 		CString sArgs(sArgStr);
@@ -131,7 +137,7 @@ public:
 			} else if (sOpt.CaseCmp("-IPV4") == 0) {
 				bIPv6 = false;
 			} else {
-				CUtils::PrintMessage("Unknown option [" + sOpt + "] valid options are -ipv4 or -ipv6", true);
+				sMessage = "Unknown option [" + sOpt + "] valid options are -ipv4 or -ipv6";
 				return false;
 			}
 		}
@@ -153,7 +159,7 @@ public:
 		}
 
 		if (!sPort.empty()) {
-			m_uPort = sPort.ToUInt();
+			m_uPort = sPort.ToUShort();
 		}
 
 		CWebAdminSock* pListenSock = new CWebAdminSock(this);
@@ -163,7 +169,11 @@ public:
 		}
 #endif
 
-		return m_pManager->ListenHost(m_uPort, "WebAdmin::Listener", m_sListenHost, bSSL, SOMAXCONN, pListenSock, 0, bIPv6);
+		bool b = m_pManager->ListenHost(m_uPort, "WebAdmin::Listener", m_sListenHost, bSSL, SOMAXCONN, pListenSock, 0, bIPv6);
+		if (!b) {
+			sMessage = "Could not bind to port " + CString(m_uPort) + ": " + CString(strerror(errno));
+		}
+		return b;
 	}
 
 	void AddSock(CWebAdminSock* pSock) {
@@ -203,7 +213,7 @@ public:
 	}
 
 private:
-	unsigned int				m_uPort;
+	unsigned short				m_uPort;
 	CString						m_sSkinName;
 	set<CWebAdminSock*>			m_spSocks;
 	CString						m_sListenHost;
@@ -805,6 +815,7 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 			m_Template["QuitMsg"] = pUser->GetQuitMsg();
 			m_Template["DefaultChanModes"] = pUser->GetDefaultChanModes();
 			m_Template["BufferCount"] = CString(pUser->GetBufferCount());
+			m_Template["TimestampFormat"] = pUser->GetTimestampFormat();
 
 			const set<CString>& ssAllowedHosts = pUser->GetAllowedHosts();
 			for (set<CString>::const_iterator it = ssAllowedHosts.begin(); it != ssAllowedHosts.end(); it++) {
@@ -917,17 +928,27 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 		o6["DisplayName"] = "Use Client IP";
 		if (pUser && pUser->UseClientIP()) { o6["Checked"] = "true"; }
 
-		if (IsAdmin()) {
-			CTemplate& o7 = m_Template.AddRow("OptionLoop");
-			o7["Name"] = "denyloadmod";
-			o7["DisplayName"] = "Deny LoadMod";
-			if (pUser && pUser->DenyLoadMod()) { o7["Checked"] = "true"; }
+		CTemplate& o7 = m_Template.AddRow("OptionLoop");
+		o7["Name"] = "appendtimestamp";
+		o7["DisplayName"] = "Append Timestamps";
+		if (pUser && pUser->GetTimestampAppend()) { o7["Checked"] = "true"; }
 
-			CTemplate& o8 = m_Template.AddRow("OptionLoop");
-			o8["Name"] = "isadmin";
-			o8["DisplayName"] = "Admin";
-			if (pUser && pUser->IsAdmin()) { o8["Checked"] = "true"; }
-			if (pUser && pUser == CZNC::Get().FindUser(GetUser())) { o8["Disabled"] = "true"; }
+		CTemplate& o8 = m_Template.AddRow("OptionLoop");
+		o8["Name"] = "prependtimestamp";
+		o8["DisplayName"] = "Prepend Timestamps";
+		if (pUser && pUser->GetTimestampPrepend()) { o8["Checked"] = "true"; }
+
+		if (IsAdmin()) {
+			CTemplate& o9 = m_Template.AddRow("OptionLoop");
+			o9["Name"] = "denyloadmod";
+			o9["DisplayName"] = "Deny LoadMod";
+			if (pUser && pUser->DenyLoadMod()) { o9["Checked"] = "true"; }
+
+			CTemplate& o10 = m_Template.AddRow("OptionLoop");
+			o10["Name"] = "isadmin";
+			o10["DisplayName"] = "Admin";
+			if (pUser && pUser->IsAdmin()) { o10["Checked"] = "true"; }
+			if (pUser && pUser == CZNC::Get().FindUser(GetUser())) { o10["Disabled"] = "true"; }
 		}
 
 		PrintPage(sPageRet, "UserPage.tmpl");
@@ -1080,6 +1101,7 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	sArg = GetParam("vhost"); if (!sArg.empty()) { pNewUser->SetVHost(sArg); }
 	sArg = GetParam("quitmsg"); if (!sArg.empty()) { pNewUser->SetQuitMsg(sArg); }
 	sArg = GetParam("chanmodes"); if (!sArg.empty()) { pNewUser->SetDefaultChanModes(sArg); }
+	sArg = GetParam("timestampformat"); if (!sArg.empty()) { pNewUser->SetTimestampFormat(sArg); }
 
 	pNewUser->SetBufferCount(GetParam("bufsize").ToUInt());
 	pNewUser->SetKeepBuffer(GetParam("keepbuffer").ToBool());
@@ -1088,6 +1110,8 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	pNewUser->SetAutoCycle(GetParam("autocycle").ToBool());
 	pNewUser->SetKeepNick(GetParam("keepnick").ToBool());
 	pNewUser->SetUseClientIP(GetParam("useclientip").ToBool());
+	pNewUser->SetTimestampAppend(GetParam("appendtimestamp").ToBool());
+	pNewUser->SetTimestampPrepend(GetParam("prependtimestamp").ToBool());
 
 	if (IsAdmin()) {
 		pNewUser->SetDenyLoadMod(GetParam("denyloadmod").ToBool());
