@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #include "znc.h"
 #include "User.h"
@@ -20,7 +26,11 @@ CZNC::CZNC() {
 	m_pModules = new CGlobalModules();
 #endif
 	m_pISpoofLockFile = NULL;
+	m_uiConnectDelay = 30;
 	SetISpoofFormat(""); // Set ISpoofFormat to default
+	m_uBytesRead = 0;
+	m_uBytesWritten = 0;
+	m_pConnectUserTimer = NULL;
 }
 
 CZNC::~CZNC() {
@@ -43,6 +53,7 @@ CZNC::~CZNC() {
 		a->second->SetBeingDeleted(true);
 	}
 
+	// This deletes m_pConnectUserTimer
 	m_Manager.Cleanup();
 	DeleteUsers();
 
@@ -53,12 +64,12 @@ CZNC::~CZNC() {
 
 CString CZNC::GetTag(bool bIncludeVersion) {
 	if (!bIncludeVersion) {
-		return "ZNC - by prozac@rottenboy.com";
+		return "ZNC by prozac - http://znc.sourceforge.net";
 	}
 
 	char szBuf[128];
 	memset(szBuf, 0, 128);
-	snprintf(szBuf, 127, "ZNC %1.3f - by prozac@rottenboy.com", VERSION);
+	snprintf(szBuf, 127, "ZNC %1.3f by prozac - http://znc.sourceforge.net", VERSION);
 
 	return szBuf;
 }
@@ -79,9 +90,55 @@ bool CZNC::OnBoot() {
 	return true;
 }
 
+bool CZNC::ConnectUser(CUser *pUser) {
+	CString sSockName = "IRC::" + pUser->GetUserName();
+	CIRCSock* pIRCSock = (CIRCSock*) m_Manager.FindSockByName(sSockName);
+
+	if (m_pISpoofLockFile != NULL) {
+		return false;
+	}
+
+
+	if (pIRCSock || !pUser->HasServers())
+		return false;
+
+	if (pUser->ConnectPaused())
+		return false;
+
+	CServer* pServer = pUser->GetNextServer();
+
+	if (!pServer)
+		return false;
+
+	if (!WriteISpoof(pUser)) {
+		DEBUG_ONLY(cout << "ISpoof could not be written" << endl);
+		pUser->PutStatus("ISpoof could not be written, retrying...");
+		return true;
+	}
+
+	DEBUG_ONLY(cout << "User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << ":" << pServer->GetPort() << "] ..." << endl);
+	pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + ":" + CString(pServer->GetPort()) + "] ...");
+
+	pIRCSock = new CIRCSock(pUser);
+	pIRCSock->SetPass(pServer->GetPass());
+
+	bool bSSL = false;
+#ifdef HAVE_LIBSSL
+	if (pServer->IsSSL()) {
+		bSSL = true;
+	}
+#endif
+
+	if (!m_Manager.Connect(pServer->GetName(), pServer->GetPort(), sSockName, 20, bSSL, pUser->GetVHost(), pIRCSock)) {
+		ReleaseISpoof();
+		pUser->PutStatus("Unable to connect. (Bad host?)");
+	}
+
+	return true;
+}
+
 int CZNC::Loop() {
-	m_Manager.SetSelectTimeout(10000);
-	m_itUserIter = m_msUsers.begin();
+	EnableConnectUser();
 
 	while (true) {
 		// Check for users that need to be deleted
@@ -108,72 +165,19 @@ int CZNC::Loop() {
 #ifdef _MODULES
 				pUser->DelModules();
 #endif
+				AddBytesRead(pUser->BytesRead());
+				AddBytesWritten(pUser->BytesWritten());
 				delete pUser;
 			}
 
 			m_ssDelUsers.clear();
-			m_itUserIter = m_msUsers.begin();
+			RestartConnectUser();
 			WriteConfig();
 		}
 
-		m_Manager.Loop();
-
-		if (m_pISpoofLockFile != NULL) {
-			continue;
-		}
-
-		if (m_itUserIter == m_msUsers.end()) {
-			m_itUserIter = m_msUsers.begin();
-		}
-
-		if (m_msUsers.empty()) {
-			usleep(10000);
-			continue;
-		}
-
-		CString sSockName = "IRC::" + m_itUserIter->first;
-		CUser* pUser = m_itUserIter->second;
-
-		m_itUserIter++;
-
-		CIRCSock* pIRCSock = (CIRCSock*) m_Manager.FindSockByName(sSockName);
-
-		if (!pIRCSock && pUser->HasServers()) {
-			if (pUser->ConnectPaused() && pUser->IsLastServer()) {
-				continue;
-			}
-
-			CServer* pServer = pUser->GetNextServer();
-
-			if (!pServer) {
-				continue;
-			}
-
-			if(!WriteISpoof(pUser)) {
-				DEBUG_ONLY(cout << "ISpoof could not be written" << endl);
-				pUser->PutStatus("ISpoof could not be written, retrying...");
-				continue;
-			}
-
-			DEBUG_ONLY(cout << "User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << ":" << pServer->GetPort() << "] ..." << endl);
-			pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + ":" + CString(pServer->GetPort()) + "] ...");
-
-			pIRCSock = new CIRCSock(pUser);
-			pIRCSock->SetPass(pServer->GetPass());
-
-			bool bSSL = false;
-#ifdef HAVE_LIBSSL
-			if (pServer->IsSSL()) {
-				bSSL = true;
-				pIRCSock->SetPemLocation(GetPemLocation());
-			}
-#endif
-
-			if (!m_Manager.Connect(pServer->GetName(), pServer->GetPort(), sSockName, 20, bSSL, pUser->GetVHost(), pIRCSock)) {
-				ReleaseISpoof();
-				pUser->PutStatus("Unable to connect. (Bad host?)");
-			}
-		}
+		// Csocket wants micro seconds
+		// between 5 to 600 secs
+		m_Manager.DynamicSelectLoop(5 * 1000 * 1000, 600 * 1000 * 1000);
 	}
 
 	return 0;
@@ -230,8 +234,16 @@ void CZNC::ReleaseISpoof() {
 
 bool CZNC::WritePidFile(int iPid) {
 	if (!m_sPidFile.empty()) {
-		CFile File(m_sPidFile);
-		CUtils::PrintAction("Writing pid file [" + m_sPidFile + "]");
+		CString sFile;
+
+		// absolute path or relative to the data dir?
+		if (m_sPidFile[0] != '/')
+			sFile = GetZNCPath() + "/" + m_sPidFile;
+		else
+			sFile = m_sPidFile;
+
+		CFile File(sFile);
+		CUtils::PrintAction("Writing pid file [" + sFile + "]");
 
 		if (File.Open(O_WRONLY | O_TRUNC | O_CREAT)) {
 			File.Write(CString(iPid) + "\n");
@@ -289,7 +301,7 @@ void CZNC::DeleteUsers() {
 	}
 
 	m_msUsers.clear();
-	m_itUserIter = m_msUsers.begin();
+	RestartConnectUser();
 }
 
 CUser* CZNC::GetUser(const CString& sUser) {
@@ -387,6 +399,8 @@ bool CZNC::WriteConfig() {
 		File.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\r\n");
 	}
 
+	File.Write("ConnectDelay = " + CString(m_uiConnectDelay) + "\r\n");
+
 	if (!m_sISpoofFile.empty()) {
 		File.Write("ISpoofFile   = " + m_sISpoofFile + "\r\n");
 		if (!m_sISpoofFormat.empty()) { File.Write("ISpoofFormat = " + m_sISpoofFormat + "\r\n"); }
@@ -986,6 +1000,37 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					} else if (sName.CaseCmp("Chan") == 0) {
 						pUser->AddChan(sValue, true);
 						continue;
+					} else if (sName.CaseCmp("TimestampFormat") == 0) {
+						pUser->SetTimestampFormat(sValue);
+						continue;
+					} else if (sName.CaseCmp("AppendTimestamp") == 0) {
+						pUser->SetTimestampAppend(sValue.ToBool());
+						continue;
+					} else if (sName.CaseCmp("PrependTimestamp") == 0) {
+						pUser->SetTimestampPrepend(sValue.ToBool());
+						continue;
+					} else if (sName.CaseCmp("Timestamp") == 0) {
+						if(sValue.Trim_n().CaseCmp("true") != 0) {
+							if(sValue.Trim_n().CaseCmp("append") == 0) {
+								pUser->SetTimestampAppend(true);
+								pUser->SetTimestampPrepend(false);
+							} else if(sValue.Trim_n().CaseCmp("prepend") == 0) {
+								pUser->SetTimestampAppend(false);
+								pUser->SetTimestampPrepend(true);
+							} else if(sValue.Trim_n().CaseCmp("false") == 0) {
+								pUser->SetTimestampAppend(false);
+								pUser->SetTimestampPrepend(false);
+							} else {
+								pUser->SetTimestampFormat(sValue);
+							}
+						}
+						continue;
+					} else if (sName.CaseCmp("TimezoneOffset") == 0) {
+						pUser->SetTimezoneOffset(sValue.ToDouble()); // there is no ToFloat()
+						continue;
+					} else if (sName.CaseCmp("JoinTries") == 0) {
+						pUser->SetJoinTries(sValue.ToUInt());
+						continue;
 					} else if (sName.CaseCmp("LoadModule") == 0) {
 						CString sModName = sValue.Token(0);
 						CUtils::PrintAction("Loading Module [" + sModName + "]");
@@ -995,7 +1040,13 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 
 						try {
 							bool bModRet = pUser->GetModules().LoadModule(sModName, sArgs, pUser, sModRet);
-							CUtils::PrintStatus(bModRet, (bModRet) ? "" : sModRet);
+
+							// If the module was loaded, sModRet contains
+							// "Loaded Module [name] ..." and we strip away this beginning.
+							if (bModRet)
+								sModRet = sModRet.Token(1, true, sModName + "] ");
+
+							CUtils::PrintStatus(bModRet, sModRet);
 							if (!bModRet) {
 								return false;
 							}
@@ -1101,7 +1152,13 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 
 					try {
 						bool bModRet = GetModules().LoadModule(sModName, sArgs, NULL, sModRet);
-						CUtils::PrintStatus(bModRet, (bModRet) ? "" : sModRet);
+
+						// If the module was loaded, sModRet contains
+						// "Loaded Module [name] ..." and we strip away this beginning.
+						if (bModRet)
+							sModRet = sModRet.Token(1, true, sModName + "] ");
+
+						CUtils::PrintStatus(bModRet, sModRet);
 						if (!bModRet) {
 							return false;
 						}
@@ -1130,16 +1187,14 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					AddVHost(sValue);
 					continue;
 				} else if (sName.CaseCmp("PidFile") == 0) {
-					if (!sValue.empty() && sValue[0] != '/') {
-						m_sPidFile = GetZNCPath() + "/" + sValue;
-					} else {
-						m_sPidFile = sValue;
-					}
-
+					m_sPidFile = sValue;
 					continue;
 				} else if (sName.CaseCmp("StatusPrefix") == 0) {
 					m_sStatusPrefix = sValue;
 					continue;
+				} else if (sName.CaseCmp("ConnectDelay") == 0) {
+					m_uiConnectDelay = sValue.ToUInt();
+					continue;
 				}
 			}
 		}
@@ -1159,6 +1214,10 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 		return false;
 	}
 
+#ifdef _MODULES
+	GetModules().OnFinishedConfig();
+#endif
+
 	return true;
 }
 
@@ -1258,3 +1317,96 @@ CZNC& CZNC::Get() {
 	return *pZNC;
 }
 
+void CZNC::UpdateTrafficStats() {
+	CSockManager* p = &m_Manager;
+	for (unsigned int a = 0; a < p->size(); a++) {
+		if ((*p)[a]->GetSockName().Left(5) == "IRC::") {
+			CIRCSock *i = (CIRCSock *)(*p)[a];
+			i->GetUser()->AddBytesRead((*p)[a]->GetBytesRead());
+			(*p)[a]->ResetBytesRead();
+			i->GetUser()->AddBytesWritten((*p)[a]->GetBytesWritten());
+			(*p)[a]->ResetBytesWritten();
+		} else if ((*p)[a]->GetSockName().Left(5) == "USR::") {
+			CClient *c = (CClient *)(*p)[a];
+			c->GetUser()->AddBytesRead((*p)[a]->GetBytesRead());
+			(*p)[a]->ResetBytesRead();
+			c->GetUser()->AddBytesWritten((*p)[a]->GetBytesWritten());
+			(*p)[a]->ResetBytesWritten();
+		}
+	}
+}
+
+class CConnectUserTimer : public CCron {
+public:
+	CConnectUserTimer(int iSecs) : CCron() {
+		SetName("Connect users");
+		Start(iSecs);
+		m_itUserIter = CZNC::Get().GetUserMap().begin();
+	}
+	virtual ~CConnectUserTimer() {}
+
+protected:
+	virtual void RunJob() {
+		unsigned int uiUserCount;
+		map<CString,CUser*>::const_iterator end;
+		bool bUsersLeft = false;
+
+		uiUserCount = CZNC::Get().GetUserMap().size();
+		end = CZNC::Get().GetUserMap().end();
+
+		// Try to connect each user, if this doesnt work, abort
+		for (unsigned int i = 0; i < uiUserCount; i++) {
+			if (m_itUserIter == end) {
+				m_itUserIter = CZNC::Get().GetUserMap().begin();
+			}
+
+			CUser* pUser = m_itUserIter->second;
+
+			m_itUserIter++;
+
+			// Is this user disconnected and does he want to connect?
+			if (pUser->GetIRCSock() == NULL && pUser->GetIRCConnectEnabled())
+				bUsersLeft = true;
+
+			if (CZNC::Get().ConnectUser(pUser))
+				// Wait until next time timer fires
+				return;
+		}
+
+		if (bUsersLeft == false)
+			CZNC::Get().DisableConnectUser();
+	}
+
+private:
+	map<CString,CUser*>::const_iterator	m_itUserIter;
+};
+
+void CZNC::EnableConnectUser() {
+	if (m_pConnectUserTimer != NULL)
+		return;
+
+	m_pConnectUserTimer = new CConnectUserTimer(m_uiConnectDelay);
+	GetManager().AddCron(m_pConnectUserTimer);
+}
+
+void CZNC::DisableConnectUser() {
+	if (m_pConnectUserTimer == NULL)
+		return;
+
+	// This will kill the cron
+	m_pConnectUserTimer->Stop();
+	m_pConnectUserTimer = NULL;
+}
+
+void CZNC::RestartConnectUser() {
+	DisableConnectUser();
+
+	map<CString, CUser*>::iterator end = m_msUsers.end();
+	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != end; it++) {
+		// If there is a user without irc socket we need the timer
+		if (it->second->GetIRCSock() == NULL) {
+			EnableConnectUser();
+			return;
+		}
+	}
+}
