@@ -1,4 +1,10 @@
-//! @author prozac@rottenboy.com
+/*
+ * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 as published
+ * by the Free Software Foundation.
+ */
 
 #ifndef _ZNC_H
 #define _ZNC_H
@@ -81,6 +87,8 @@ private:
 protected:
 };
 
+class CConnectUserTimer;
+
 class CZNC {
 public:
 	CZNC();
@@ -106,6 +114,11 @@ public:
 	bool AddVHost(const CString& sHost);
 	bool RemVHost(const CString& sHost);
 	void Broadcast(const CString& sMessage, CUser* pUser = NULL);
+	void AddBytesRead(unsigned long long u) { m_uBytesRead += u; }
+	void AddBytesWritten(unsigned long long u) { m_uBytesWritten += u; }
+	unsigned long long BytesRead() const { return m_uBytesRead; }
+	unsigned long long BytesWritten() const { return m_uBytesWritten; }
+	void UpdateTrafficStats();
 
 	// Setters
 	void SetStatusPrefix(const CString& s) { m_sStatusPrefix = (s.empty()) ? "*" : s; }
@@ -149,6 +162,14 @@ public:
 	const VCString& GetMotd() const { return m_vsMotd; }
 	// !MOTD
 
+	// Create a CIRCSocket. Return false if user cant connect
+	bool ConnectUser(CUser *pUser);
+	// This creates a CConnectUserTimer if we haven't got one yet
+	void EnableConnectUser();
+	void DisableConnectUser();
+	// This needs to be called if anything was added / removed to m_msUsers
+	void RestartConnectUser();
+
 private:
 protected:
 	vector<CListener*>		m_vpListeners;
@@ -174,10 +195,13 @@ protected:
 	VCString				m_vsMotd;
 	CLockFile				m_LockFile;
 	CLockFile*				m_pISpoofLockFile;
-	map<CString,CUser*>::iterator	m_itUserIter;	// This needs to be reset to m_msUsers.begin() if anything is added or removed to the map
+	uint					m_uiConnectDelay;
 #ifdef _MODULES
 	CGlobalModules*			m_pModules;
 #endif
+	unsigned long long		m_uBytesRead;
+	unsigned long long		m_uBytesWritten;
+	CConnectUserTimer		*m_pConnectUserTimer;
 };
 
 class CListener {
